<?php

// Includes the startup functions file
require get_template_directory() . '/inc/pema-on-startup.php';

// Includes the on switch files, e.g., when the theme is deactivated
require get_template_directory() . '/inc/pema-on-switch.php';

// Includes all the ajax php files
require get_template_directory() . '/inc/pema-ajax.php';



function my_scripts() {
	global $current_user;
	$current_user = wp_get_current_user();

	// Registers and loads script.js file
	wp_register_script('script', get_template_directory_uri() . '/js/script.js', array( 'jquery' ));
	wp_enqueue_script( 'script');

	// Localizes variables for use in AJAX
	wp_localize_script( 'script' , 'ajax_params', array(
		'ajaxurl' => admin_url( 'admin-ajax.php' ),
		'loggedinuserID' => $current_user->ID,
		'ajax_nonce' => wp_create_nonce('pema_ajax_nonce'),
		) );

	// Registers and loads style.css
	wp_register_style('style', get_stylesheet_uri());
	wp_enqueue_style( 'style');

		// Adding jquery-confirm plugin for jquery.
	wp_register_style('jquery-confirmcss', 'https://cdnjs.cloudflare.com/ajax/libs/jquery-confirm/3.3.2/jquery-confirm.min.css');
	wp_register_script('jquery-confirm', 'https://cdnjs.cloudflare.com/ajax/libs/jquery-confirm/3.3.2/jquery-confirm.min.js', array('pema_admin_script'));
	wp_enqueue_script('jquery-confirm');
	wp_enqueue_style('jquery-confirmcss');

}

// Runs my_scripts()
add_action('wp_enqueue_scripts', 'my_scripts');


/* ------------- ADD CSS FILE TO ADMIN PAGE ------------- */

function pema_load_admin_styles( $hook ) {
	//wp_die($hook);	//Used to see what the $hook is

	// Gets the current user
	global $current_user;
	$current_user = wp_get_current_user();

	// Registers and then loads global CSS styles
	wp_register_style('pema_admin_style', get_template_directory_uri() . '/css/pema_admin_style.css');
	wp_enqueue_style('pema_admin_style');

	// Registers and loads functions used by multiple JS files
	wp_register_script('pema_admin_script', get_template_directory_uri() . '/js/pema_admin_script.js', array( 'jquery'));
	wp_enqueue_script( 'pema_admin_script');

	// Localizes variables for use in AJAX
	wp_localize_script( 'pema_admin_script' , 'ajax_params', array(
		'ajaxurl' => admin_url( 'admin-ajax.php' ),
		'loggedinuserID' => $current_user->ID,
		'ajax_nonce' => wp_create_nonce('pema_ajax_nonce'),
		) );

	// Goes through all JS files in the js-admin directory and registers them for use
	// The array is dependencies, i.e., it will only load after jquery and pema_admin_script
	foreach ( glob( get_template_directory() . '/js-admin/*.js' ) as $file ) {
		$file = str_replace(get_template_directory() . '/js-admin/', '', $file);
		wp_register_script( $file, get_template_directory_uri() . '/js-admin/' . $file, array('jquery', 'pema_admin_script'));
	}

	// Goes through all CSS files in the css-admin directory and registers them for use
	// The array is dependencies, i.e., it will only load after pema_admin_style
	foreach ( glob( get_template_directory() . '/css-admin/*.css' ) as $file ) {
		$file = str_replace(get_template_directory() . '/css-admin/', '', $file);
		wp_register_style( $file, get_template_directory_uri() . '/css-admin/' . $file, array('pema_admin_style'));
	}


	// Checks what page the user is on, then only loads the corresponding JS and CSS files

	// Create Labs page
	if ( 'manage-labs_page_pema_create_labs' === $hook ) {
		wp_enqueue_script('create_lab.js');
		wp_enqueue_style('create_lab.css');

	// Manage tasks for lab page
	} elseif ( 'manage-labs_page_pema_manage_tasks_lab' === $hook ) {
		wp_enqueue_script('manage_tasks_for_lab.js');
		wp_enqueue_style('manage_tasks_for_lab.css');

	// Edit lab page
	} elseif ( 'manage-labs_page_pema_edit_lab' === $hook ) {
		wp_enqueue_script('edit_lab.js');
		wp_enqueue_style('edit_lab.css');

	// Edit group page
	} elseif ( 'users_page_pema_edit_groups' === $hook ) {
		wp_enqueue_script('edit_group.js');
		wp_enqueue_style('edit_group.css');

	// Manage topcis page
	} elseif ( 'toplevel_page_pema_manage_topics' === $hook ) {
		wp_enqueue_script('manage_topics.js');
		wp_enqueue_style('manage_topics.css');

	// Create group page
	} elseif ( 'users_page_pema_create_groups' === $hook ) {
		wp_enqueue_script('create_group.js');
		wp_enqueue_style('create_group.css');

	// Edit tasks page
	} elseif ( 'manage-task-templates_page_pema_edit_tasks' === $hook ) {
		wp_enqueue_script('edit_task.js');
		wp_enqueue_style('edit_task.css');

	// Create task page
	} elseif ( 'manage-task-templates_page_pema_create_tasks' === $hook ) {
		wp_enqueue_script('create_task.js');
		wp_enqueue_style('create_task.css');

	// Manage tasks page
	} elseif ( 'toplevel_page_pema_manage_tasks' === $hook ) {
		wp_enqueue_script('manage_tasks.js');
		wp_enqueue_style('manage_task.css');

	// Create post/news page
	} elseif ( 'toplevel_page_include-text' === $hook ) {
		wp_enqueue_script('post_news.js');
		wp_enqueue_style('post_news.css');

	// Manage labs page
	} elseif ( 'toplevel_page_pema_manage_labs' === $hook  ) {
		wp_enqueue_script('manage_lab.js');
		wp_enqueue_style('manage_lab.css');

	// Submitted tasks page
	} elseif ( 'toplevel_page_pema_submitted_tasks' === $hook ) {
		wp_enqueue_script('submitted_tasks.js');
		wp_enqueue_style('submitted_tasks.css');
	}


	// Registers and loads the chosen jQuery plugin
    wp_register_style('chosencss', 'https://cdnjs.cloudflare.com/ajax/libs/chosen/1.8.7/chosen.min.css');
    wp_register_script('chosenjs', 'https://cdnjs.cloudflare.com/ajax/libs/chosen/1.8.7/chosen.jquery.min.js', array('pema_admin_script'));
    wp_enqueue_script('chosenjs');
    wp_enqueue_style('chosencss');
    //Look here for most recent releases of chosen https://cdnjs.com/libraries/chosen

    // Registers and loads the flatpickr jQuery plugin, along with the plugin of the flatpickr plugin
    wp_register_script('flatpickr', 'https://cdn.jsdelivr.net/npm/flatpickr', array('pema_admin_script'));
    wp_register_style('flatpickrcss', 'https://npmcdn.com/flatpickr/dist/themes/dark.css');
    wp_register_script('flatpickrrangeplugin', 'https://cdnjs.cloudflare.com/ajax/libs/flatpickr/4.5.4/plugins/rangePlugin.js', array('pema_admin_script'));
    wp_enqueue_script('flatpickrrangeplugin');
    wp_enqueue_script('flatpickr');
    wp_enqueue_style('flatpickrcss');

    // Registers and loads the sortable jQuery plugin
	wp_register_script('sortable', 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.10.3/jquery-ui.min.js', array('pema_admin_script'));
	wp_enqueue_script('sortable');

	// Adding jquery-confirm plugin for jquery.
	wp_register_style('jquery-confirmcss', 'https://cdnjs.cloudflare.com/ajax/libs/jquery-confirm/3.3.2/jquery-confirm.min.css');
	wp_register_script('jquery-confirm', 'https://cdnjs.cloudflare.com/ajax/libs/jquery-confirm/3.3.2/jquery-confirm.min.js', array('pema_admin_script'));
	wp_enqueue_script('jquery-confirm');
	wp_enqueue_style('jquery-confirmcss');
	// Jquery Confirm 3.3.2 from: https://craftpip.github.io/jquery-confirm/

	// Registers and loads the multiselect jQuery plugin
	wp_register_script('jquery-multiselect', 'https://cdnjs.cloudflare.com/ajax/libs/multi-select/0.9.12/js/jquery.multi-select.min.js', array('pema_admin_script'));
	wp_enqueue_script('jquery-multiselect');
	wp_register_style('jquery-multiselectcss', 'https://cdnjs.cloudflare.com/ajax/libs/multi-select/0.9.12/css/multi-select.min.css');
	wp_enqueue_style('jquery-multiselectcss');

}

// When admin page is accessed
add_action('admin_enqueue_scripts', 'pema_load_admin_styles');


/* ---- Styling the WordPress-login ---- */
function my_login_logo() { 
	?>
    <style type="text/css">
        #login h1 a, .login h1 a {
        background-image: url(<?php echo get_template_directory_uri(); ?>/NTNU-logo.png);
		height:70px;
		width:320px;
		background-size: 320px 62px;
		background-repeat: no-repeat;
        padding-bottom: 30px;
        }
    </style>
	<?php 
}
add_action( 'login_enqueue_scripts', 'my_login_logo' );


/* ------------- REMOVES ADMIN BAR ------------- */

// This function removes the wordpress "admin bar"
function pema_remove_admin_bar()
{
    return false;
}
// Remove Admin bar instead of displaying.
add_filter('show_admin_bar', 'pema_remove_admin_bar');




/* ------------- DISPLAY LOGGED IN USER ON FRONT PAGE ------------- */

// Function for displaying logged in user
function pema_display_logged_in_user() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "general-view/loggedin.php";

    if ( file_exists( $file ) )
        require $file;

}


/* ------------- WHEN A USER IS CREATED ------------- */

// When a user is created
add_action('user_register', 'pema_on_user_create');

function pema_on_user_create($userID) {
	// Get user info from the userID
	$user_info = get_userdata($userID);
	// Get the roles of the user
	$user_roles = $user_info->roles;

	// If the user has the role Student, add metadata "show" (used for showing profile info)
	if ( in_array('Student', $user_roles) ) {
		add_user_meta($userID, 'pema-hide', 'show', true);
	// Else, i.e., user is instructor/admin, add metadata "hide" (used for hiding profile info)
	} else {
		add_user_meta($userID, 'pema-hide', 'hide', true);
	}
}


/* ------- STOPPING INSTRUCTOR FROM CREATING/BECOMING/DELETING ADMIN  ------- */

/* --- Hide Administrator From User List --- */
function pema_pre_user_query( $user_search ) {
	// If current user is not administrator - Remove Administrators from list
    if ( !current_user_can( 'administrator' ) ) {
        global $wpdb;

        // Calls string-replace for array.
        $user_search->query_where = str_replace(
        	// Search term (Always true).
            'WHERE 1=1',
            "WHERE 1=1 AND {$wpdb->users}.ID IN (
              SELECT {$wpdb->usermeta}.user_id FROM $wpdb->usermeta
              WHERE {$wpdb->usermeta}.meta_key = '{$wpdb->prefix}capabilities'
              AND {$wpdb->usermeta}.meta_value NOT LIKE '%administrator%' )",
              // Replacement value. Puts every user (group?) back in, as long as not admin.
              // The "subject"/target.
            $user_search->query_where
        );
    }
}

// Adds function call via hook. Called when someone wants to show all users.
add_action( 'pre_user_query', 'pema_pre_user_query' );


/* --- Removes an Instructors ability to add/promote (to) admin and instructor accounts --- */
// Function declaration.
function pema_editable_users_filter( $roles ) {
	// Gets current user information from database.
    $current_user = wp_get_current_user();

    // If current user is an instructor.
    if ( in_array( 'instructor', $current_user->roles ) ) {
    	// Unsets the administrator from the array editable_roles array.
        unset( $roles['administrator'] );
        // Removes instructor from list.
		unset( $roles['instructor']);
    }

    // Returns array $roles.
    return $roles;
}
// Function call via hook.
add_filter( 'editable_roles', 'pema_editable_users_filter' );


/* --- Blocks users from deleting administrator accounts --- */
// Hooks onto the "delete_user" action.
add_action('delete_user', function($id) {
	// Gets user metadata for selected user.
	$user_meta=get_userdata($id);
	// Creates an array of all roles the user is part of.
	$user_roles=$user_meta->roles;
	// Get current user information.
	$current_user = wp_get_current_user();


	// Checks if any of the roles listed are "administrator"
	if ( in_array( 'administrator', $user_roles, true )) {
		// If yes, stops deletion and gives error message.
		die("You are not allowed to delete admins!");
	}

	// If the current user is an instructor.
	if ( in_array( 'instructor', $current_user->roles ) ) {
		// If user being deleted is an instructor.
		if ( in_array( 'instructor', $user_roles, true) ) {
			// Give error. This is a dirty way to do this, but works for a proof of concept.
			wp_die("You are not allowed to delete other instructors!");
		}
	}
});


/* ------------ ADDING CUSTOM PAGES TO WP ADMIN-DASH ------------ */

// Adding page for post creation.

add_action( 'admin_menu', 'pema_createPost_register' );
function pema_createPost_register() {
	// Adding menu for post creation.
    add_menu_page(
    	// page title
        'Newsfeed Posts',
        // menu title
        'Post to Newsfeed',
        // capability
        'pema_create_post',
        // menu slug
        'include-text',
        // callback function
        'pema_createPost_render',
        // Icon
		'dashicons-testimonial'
    );
}

// Adding pages for lab management and creation.

add_action( 'admin_menu', 'pema_labs_register' );
function pema_labs_register() {
	// Adding menu entry for
	add_menu_page(
		// page title
		'Lab View',
		// menu title
		'Manage labs',
		// capability
		'pema_view_lab',
		// menu slug
		'pema_manage_labs',
		// callback function
		'pema_view_lab',
		// Icon
		'dashicons-desktop'
	);

	// Adding submenu for lab-creation.
	add_submenu_page(
		// Parent menu slug.
		'pema_manage_labs',
		// Page title
		'Create lab',
		// Menu title
		'Create lab',
		// Capability
		'pema_create_lab',
		// Menu slug
		'pema_create_labs',
		// Callback function
		'pema_create_lab'
	);

	add_submenu_page(
		// Parent menu slug.
		'pema_manage_labs',
		// Page title
		'Edit lab',
		// Menu title
		'Edit lab',
		// Capability
		'pema_edit_lab',
		// Menu slug
		'pema_edit_lab',
		// Callback function
		'pema_edit_lab'
	);

	add_submenu_page(
		// Parent menu slug.
		'pema_manage_labs',
		// Page title
		'Tasks for lab',
		// Menu title
		'Tasks for lab',
		// Capability
		'pema_manage_tasks_lab',
		// Menu slug
		'pema_manage_tasks_lab',
		// Callback function
		'pema_manage_tasks_lab'
	);

}

// Adding pages for task template management and creation.

add_action( 'admin_menu', 'pema_task_register' );
function pema_task_register() {
	// Adding menu entry for tasks (management)
	add_menu_page(
		// Page title.
		'Manage Task Templates',
		// Menu title.
		'Manage Task Templates',
		// Capability.
		'pema_manage_task',
		// Menu slug.
		'pema_manage_tasks',
		// Callback Function.
		'pema_manageTask_render',
		// Icon.
		'dashicons-index-card'
	);

	// Adding submenu entry for task template creation.
	add_submenu_page(
		// Parent menu slug.
		'pema_manage_tasks',
		// Page title.
		'Create Task Template',
		// Menu title.
		'Create Task Template',
		// Capability.
		'pema_create_task',
		// Menu slug.
		'pema_create_tasks',
		// Callback function.
		'pema_createTask_render'
	);

	// Adding submenu entry for task template edits.
	add_submenu_page(
		// Parent menu slug.
		'pema_manage_tasks',
		// Page title.
		'Edit Task Templates',
		// Menu title.
		'Edit Task Template',
		// Capability.
		'pema_create_task',
		// Menu slug.
		'pema_edit_tasks',
		// Callback function.
		'pema_editTask_render'
	);
}

// Adding submenu entry for group management.

add_action('admin_menu', 'pema_group_register');
function pema_group_register() {
	// Adding submenu entry for topic management.
	add_submenu_page(
		// Parent slug
    	'users.php',
    	// Page title
    	'Create group',
    	// Menu title
    	'Create group',
    	// Capability
    	'pema_create_group',
    	// Menu slug
    	'pema_create_groups',
    	// callback function
    	'pema_create_group'
    );
    add_submenu_page(
    	// Parent slug
    	'users.php',
    	// Page title
    	'Edit group',
    	// Menu title
    	'Edit group',
    	// Capability
    	'pema_edit_group',
    	// Menu slug
    	'pema_edit_groups',
    	// callback function
    	'pema_edit_group'
    );
}

// Adding menu entry for topic management.

add_action('admin_menu', 'pema_topic_register');
function pema_topic_register() {
	// Adding menu entry for topic management.
	add_menu_page(
		// Page Title.
		'Manage Topics',
		// Menu Title.
		'Manage Topics',
		// Capability.
		'pema_manage_topic',
		// Menu slug.
		'pema_manage_topics',
		// Callback function.
		'pema_manageTopic_render',
		// Icon.
		'dashicons-welcome-learn-more'
	);
}

// Adding menu for seeing whos submitted tasks on a lab
add_action('admin_menu', 'pema_submitted_tasks_register');
function pema_submitted_tasks_register() {
	add_menu_page(
		// Page title
		'Submitted tasks',
		// Menu Title
		'Submitted tasks',
		// Capability
		'pema_submitted_task',
		// Menu slug
		'pema_submitted_tasks',
		// Callback function
		'pema_submitted_task',
		// Icon
		'dashicons-book-alt'
	);
}


// Actually displays the page
function pema_createPost_render() {

    global $title;

    print '<div class="wrap">';

    // Code being displayed from file ...
    $file = plugin_dir_path( __FILE__ ) . "admin-view/createPost.php";

    if ( file_exists( $file ) )
        require $file;

    print '</div>';
}

// Displays the createTask page from createTask.php
function pema_manageTask_render() {

    print '<div class="wrap">';

    // Code being displayed from file ...
    $file = plugin_dir_path( __FILE__ ) . "admin-view/manageTask.php";

    if ( file_exists( $file ) )
        require $file;

    print '</div>';
}

// Displays the createTask page from createTask.php
function pema_createTask_render() {

    print '<div class="wrap">';

    // Code being displayed from file ...
    $file = plugin_dir_path( __FILE__ ) . "admin-view/createTask.php";

    if ( file_exists( $file ) )
        require $file;

    print '</div>';
}

// Displays the editTask page from editTask.php
function pema_editTask_render() {

    print '<div class="wrap">';

    // Code being displayed from file ...
    $file = plugin_dir_path( __FILE__ ) . "admin-view/editTask.php";

    if ( file_exists( $file ) )
        require $file;

    print '</div>';
}

function pema_create_group() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/createGroup.php";

    if ( file_exists( $file ) )
        require $file;
}


function pema_view_lab() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/manageLab.php";

    if ( file_exists( $file ) )
        require $file;
}


//Display of page "create lab"
function pema_create_lab() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/createLab.php";

    if ( file_exists( $file ) )
        require $file;

}


function pema_edit_lab() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/editLab.php";

    if ( file_exists( $file ) )
        require $file;
}


function pema_manageTopic_render() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/manageTopic.php";

	if ( file_exists( $file ) )
		require $file;
}

function pema_manage_tasks_lab() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/manageTasksForLab.php";

	if ( file_exists( $file ) )
		require $file;
}

function pema_edit_group() {
	// Code being displayed from file ...
	$file = plugin_dir_path( __FILE__ ) . "admin-view/editGroup.php";

	if ( file_exists( $file ) )
		require $file;
}

function pema_submitted_task() {
	$file = plugin_dir_path(__FILE__) . "admin-view/submittedTasks.php";
	if ( file_exists( $file ) )
		require $file;
}


/* --------- Functions for hiding admin-menu entries ---------*/

// Hides top-level menus:
// Custom function for hiding top-level menu entries.
function custom_menu_page_removing() {
	// Hides the "Comments" page.
	remove_menu_page( 'edit-comments.php' );
	// Hides the "Media" page.
	remove_menu_page( 'upload.php' );
}
//Adding action to admin_menu.
add_action( 'admin_menu', 'custom_menu_page_removing' );


// Hides wordpress default post-creation tool (Submenu-item):
// Custom func for submenu-entry removal..
function custom_submenu_page_removing() {
	// Hides the "New Post" submenu entry under "Posts".
	remove_submenu_page( 'edit.php', 'post-new.php' );
}
// Adding action to admin_menu.
add_action( 'admin_menu', 'custom_submenu_page_removing');
?>
