<?php
/*


	============================================
				AJAX FUNCTIONS
	============================================

*/

// Enable to function for privileged users, e.g., logged in users
add_action('wp_ajax_pema_update_user_metadata', 'pema_update_user_metadata');			

/* -------------- UPDATES USER METADATA ------------ */

function pema_update_user_metadata() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Gets the value for the meta that will be updated
	$value = $_POST["value"];
	// Gets the user ID from ajax
	$user_id = $_POST["id"];
	// Gets the meta for the value that will be updated
	$meta = $_POST["meta"];

	// Updates the meta data for a user
	update_user_meta($user_id, $meta, $value);

	// IMPORTANT, always end a ajax function with a die()
	die();

}


/* -------------- CHECK ANSWER HASH ------------ */

// Enable the function for privileged users
add_action('wp_ajax_pema_check_hash', 'pema_check_hash');

// Function for checking usersubmitted hash
function pema_check_hash() {
	// Nonce
	check_ajax_referer('pema_ajax_nonce', 'security');

	//Initialize DB
	global $wpdb;

	// Get the user submitted hash
	$answer = $_POST['answer'];
	// Get the taskID	
	$taskID = $_POST['taskid'];
	// Get the userID
	$userID = $_POST['userid'];

	//Get the hash for the task
	$task = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_task WHERE taskID = %d", $taskID));
	//If the hash is correct
	if ( $task->taskHash === $answer ) {
		//Match
		// Get the group
		$group = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_group WHERE groupID = (SELECT groupID FROM pema_user_has_group WHERE userID = %d)", $userID));
		// If the group exist
		if ( NULL !== $group ) {
			// Insert into DB
			$inserted = $wpdb->insert(
				// Table name
				'pema_task_complete',
				// Columns and values to insert
				array(
					'taskID' => $taskID,
					'groupID' => $group->groupID
				),
				// Array in sprintf-like format
				// %d for INT, %s for STRING, %f for FLOAT
				array(
					'%d',
					'%d'
				)
			);
			// If error inserting, die with error message
			if ( $inserted === false ) {
				wp_die('DBERR');
			}
			echo "Correct";
		} else {
			echo "DBERR";
		}
	// Else, hash is wrong
	} else {
		echo "Wrong";
	}

	// Custom hook, allows a plugin to inject code here
	do_action('pema_after_check_hash', $answer, $taskID, $userID);

	// Die and send whatever was echoed
	die();
}


/* -------------- CREATES A NEW LAB -------------- */
// Enable function
add_action('wp_ajax_pema_deploy_lab', 'pema_deploy_lab');
// Function for creating a lab
function pema_deploy_lab() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	global $wpdb;			//Initialize DB connection

	// Get all values from AJAX, don't need to sql escape
	$title = $_POST["title"];
	$description = $_POST["description"];
	$startdate = $_POST["startdate"];
	$enddate =$_POST["enddate"];
	$usetemplate = $_POST["usetemplate"];
	$id = $_POST["id"];
	$groups = $_POST["groups"];

	// VALUES INSERTED INTO THE FIRST ARRAY SHOULD BE RAW, AS THE INSERT FUNCTION ESCAPES THEM
	// Insert function also does prepared statements as noted by the second array (%s = string, %d = int, %f = float)
	// For more info see https://codex.wordpress.org/Class_Reference/wpdb#INSERT_rows

	// If use a preexisting templates
	if ( $usetemplate == 'true' ) {
		$inserted = $wpdb->insert(
			// Table name
			'pema_lab',
			// Values to insert
			array(
				'labName' => $title,
				'labDescription' => $description,
				'labStart' => $startdate,
				'labEnd' => $enddate
			),
			array(
				'%s',
				'%s',
				'%s',
				'%s'
			)
		);
		// If error inserting, die
		if ( $inserted === false ) {
			die();
		}
	// Elseif, check if the lab name already exist as a template name
	} elseif ( $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_lab_template WHERE templateName = %s", $title)) ) {
		// If it does, die with message
		$return = array('text' => 'Exist');
		echo json_encode($return);
		die();
	// Else
	} else {
		// Inserts into table 'pema_lab_template'
		$inserted = $wpdb->insert(
			'pema_lab_template',
			array(
				'templateName' => $title,
				'templateDescription' => $description
			),
			array(
				'%s',
				'%s'
			)
		);
		// If error inserting, die
		if ( $inserted === false ) {
			die();
		}

		// Inserts into table 'pema_lab'
		$inserted = $wpdb->insert(
			'pema_lab',
			array(
				'labName' => $title,
				'labDescription' => $description,
				'labStart' => $startdate,
				'labEnd' => $enddate
			),
			array(
				'%s',
				'%s',
				'%s',
				'%s'
			)
		);
		// If error inserting, die
		if ( $inserted === false ) {
			die();
		}
	}

	// Gets ID of newly inserted table
	$lab_get_inserted_id = $wpdb->insert_id;

	// If groups are to be added
	if ( $groups ) {
		// Go through all groups
		foreach( $groups as $group ) {
			// Insert into the DB
			$inserted = $wpdb->insert(
				'pema_group_has_lab',
				array(
					'groupID' => $group,
					'labID' => $lab_get_inserted_id
				),
				array(
					'%d',
					'%d'
				)
			);
			// If error inserting, die
			if ( $inserted === false ) {
				die();
			}
		}
	}

	// Set arrays for creating a new page
	// For more info about options to insert see:
	// https://developer.wordpress.org/reference/functions/wp_insert_post/
	$newPage = array(
		'post_title' => $title,
		'post_content' => $description,
		'post_status' => 'publish',
		'post_type' => 'page',
		'post_author' => $id,
		'meta_input' => array(
			'labID' => $lab_get_inserted_id,
		)
	);

	// Insert a new page, and get the page ID
	$pageID = wp_insert_post($newPage);

	// Custom hook
	do_action('pema_after_deploy_lab', $lab_get_inserted_id, $groups, $pageID);
	

	$return = array(
		'text' => 'Success',
		'labid' => $lab_get_inserted_id
	);
	echo json_encode($return);

	// End with die()
	die();
}

/* -------------- CREATES A NEW GROUP -------------- */
// Make available for logged in users
add_action('wp_ajax_pema_ajax_create_group', 'pema_ajax_create_group');

function pema_ajax_create_group() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize Database
	global $wpdb;

	//Set variables given through ajax
	$users = $_POST['users'];
	$description = $_POST['description'];
	$title = $_POST['title'];
	$proficiency = $_POST['proficiency'];

	//Insert to database
	$inserted = $wpdb->insert(
		'pema_group',
		array(
			'proficiencyID' => $proficiency,
			'groupName' => $title,
			'groupDescription' => $description
		),
		array(
			'%d',
			'%s',
			'%s'
		)
	);
	// If error inserting, die with message
	if ( $inserted === false ) {
		wp_die('Error');
	}

	// Get the newly inserted ID in table pema_group
	$group_get_inserted_id = $wpdb->insert_id;

	// For all users
	foreach ( $users as $user ) {
		// Insert to database
		$inserted = $wpdb->insert(
			'pema_user_has_group',
			array(
				'userID' => $user,
				'groupID' => $group_get_inserted_id
			),
			array(
				'%d',
				'%d'
			)
		);
		// If error inserting, die with message
		if ( $inserted === false ) {
			wp_die('Error');
		}
	}

	// Custom hook
	do_action('pema_after_create_group', $group_get_inserted_id, $users);

	die();
}

/* -------------- DELETES A GROUP -------------- */

// Make AJAX be able to run function
add_action('wp_ajax_pema_delete_group', 'pema_delete_group');

function pema_delete_group() {
	// Check nonce
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize DB
	global $wpdb;

	// Get groupID to delete
	$id = $_POST["id"];

	// Check if the group is still a part of an active lab
	$results = $wpdb->get_results($wpdb->prepare("SELECT groupID FROM pema_group_has_lab WHERE groupID = %d", $id));
	if ( $results ) {
		wp_die('FK');
	}

	// Group VM should be deleted when a lab is deleted
	$results = $wpdb->get_results($wpdb->prepare("SELECT groupID FROM pema_group_vm WHERE groupID = %d", $id));
	if ( $results ) {
		wp_die('Error');		
	}
	// Task completed should be deleted when a task is removed from lab
	$results = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_task_complete WHERE groupID = %d", $id));
	if ( $results ) {
		wp_die('Error');
	}
	// Deletes from pema_user_has_group
	$deleted = $wpdb->delete('pema_user_has_group', array('groupID' => $id), array('%d'));
	if ( $deleted === false ) {
		wp_die('Error');
	}

	// Custom hook
	do_action('pema_before_group_delete', $id);

	// Delete the actual group
	$deleted = $wpdb->delete('pema_group', array('groupID' => $id), array('%d'));
	if ( $deleted === false ) {
		wp_die('Error');
	}
	// echo the redirect URL
	echo get_admin_url() . 'users.php?page=pema_edit_groups';
	die();
}


/* -------------- DELETES A LAB -------------- */

//Enable function for AJAX
add_action('wp_ajax_pema_delete_lab', 'pema_delete_lab');

function pema_delete_lab() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	//Initialize DB connection
	global $wpdb;

	// Gets ID from ajax
	$idcol = $_POST["idcol"];

	// Gets all pages created for the current labID
	$posts = $wpdb->get_results($wpdb->prepare("SELECT * FROM wp_posts WHERE ID IN (SELECT post_id FROM wp_postmeta WHERE meta_key = 'labID' AND meta_value = %d)", $idcol));

	// Check for Foreign Key Constraint by getting a task with the labID (only need to return one)
	$check = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_task WHERE labID = %d", $idcol));

	// If there is a constraint, die with message
	if ( $check !== NULL ) {
		wp_die('Constraint');
	}

	// Go through all pages
	foreach ( $posts as $post ) {
		// Delete page, This also deletes any post_metadata on the relevant ID
		wp_delete_post($post->ID, true);
	}
	// Delete the groupVM if it has any
	$deleted = $wpdb->delete('pema_group_VM', array('labID' => $idcol), array('%d'));

	// If error deleting, die with message
	if ( $deleted === false ) {
		wp_die('Error');
	}

	// Delete the connection between group and the lab
	$deleted = $wpdb->delete('pema_group_has_lab', array('labID' => $idcol), array('%d'));

	// If error deleting, die with message
	if ( $deleted === false ) {
		wp_die('Error');
	}

	// Custom hook
	do_action('pema_before_lab_delete', $idcol);

	// Delets the lab
	$deleted = $wpdb->delete('pema_lab', array( 'labID' => $idcol), array('%d'));
	// If error deleting, die with message
	if ( $deleted === false ) {
		wp_die('Error');
	}

	die();
}

/* ------------- Creates a post --------------- */

// Adding action for function...
add_action('wp_ajax_pema_create_post', 'pema_create_post');

// Function for post-creation.
function pema_create_post() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	$post = array(
		// Gathering the post-title
		'post_title'	=> $_POST['title1'],
		// Gathering the post-conten
		'post_content'	=> $_POST['content1'],
		// Setting the post status as "published".
		'post_status' 	=> 'publish',
		// Setting the author using the current logged-in users ID.
		'post_author'	=> $_POST['id'],
		// Specifying the post type.
		'post_type'		=> 'post'
	);

	// Calling a wordpress function for creating a post, with post array.
	$postID = wp_insert_post( $post );

	// Custom hook
	do_action('pema_after_insert_post', $postID);

	// Ends the ajax function.
	die();
}

/* ---------------- Creates a task template ---------------- */

// Adding action for function.
add_action('wp_ajax_pema_create_task_template', 'pema_create_task_template');

function pema_create_task_template() {

	// Checks the nonce, and will automatically kill the connection if false.
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initializing database connection.
	global $wpdb;

	// Getting the data from AJAX.
	$title =		 $_POST["title"];
	$description =	 $_POST["description"];
	$topic =		 $_POST["topic"];
	$useAnswer = 	 $_POST["useAnswer"];
	$answerType = 	 $_POST["answerType"];
	$proficiencies = $_POST["proficiencies"];

	// If task template is not to be used, set it to NULL
	if ( $useAnswer !== "true" ) {
		$answerType = NULL;
	}

	// Insert into database.
	$inserted = $wpdb->insert(
		'pema_task_template',
		array(
			'templateName' => $title,
			'templateDescription' => $description,
			'topic' => $topic,
			'answerType' => $answerType
		),
		array(
			'%s',
			'%s',
			'%d',
			'%d'
		)
	);
	// If error inserting, die with message
	if ( $inserted === false ) {
		wp_die('Error');
	}

	// Get newly inserted ID
	$templateID = $wpdb->insert_id;

	// For all proficiencies, e.g., beginner or experienced
	foreach ( $proficiencies as $proficiency ) {
		// Insert into pema_hint
		$inserted = $wpdb->insert(
			'pema_hint',
			array(
				'proficiencyID' => $proficiency['id'],
				'hintDescription' => $proficiency['value']
			),
			array(
				'%d',
				'%s'
			)
		);
		// If error inserting, die with message
		if ( $inserted === false ) {
			wp_die('Error');
		}
		// Get newly inserted ID
		$hintID = $wpdb->insert_id;

		// Insert into DB
		$inserted = $wpdb->insert(
			'pema_hint_helper_template',
			array(
				'hintID' => $hintID,
				'templateID' => $templateID
			),
			array(
				'%d',
				'%d'
			)
		);
		// If error inserting, die with message
		if ( $inserted === false ) {
			wp_die('Error');
		}
	}

	// Custom hook
	do_action('pema_after_insert_task_template', $templateID);

	die();
}

/* ---------------- Add tasks to lab ---------------- */

// Make AJAX be able to run function
add_action('wp_ajax_pema_ajax_add_task_to_lab', 'pema_ajax_add_task_to_lab');

function pema_ajax_add_task_to_lab() {

	// Checks the nonce, this will exit with a die() if returned false
	// https://codex.wordpress.org/Function_Reference/check_ajax_referer
	check_ajax_referer('pema_ajax_nonce', 'security');

	global $wpdb;
	// Get bool and taskID
	$hidden = $_POST["hidden"];
	$taskID = $_POST["taskID"];

	// If hidden is equal true, i.e., if task is not to be deleted
	if ( $hidden === 'true' ) {

		// Get all fields from AJAX
		$labID = $_POST["labid"];
		$orderid = $_POST["orderid"];
		$templateid = $_POST["templateid"];
		$start = $_POST["startdate"];
		$end = $_POST["enddate"];
		$id = $_POST["id"];
		$hash = NULL;

		// If $end is empty, set value to NULL
		if ( !$end ) {
			$end = NULL;
		}
		// If $start is empty, set value to NULL
		if ( !$start ) {
			$start = NULL;
		}
		// If $taskID is INT, i.e., it already exists and should be updated
		if ( filter_var($taskID, FILTER_VALIDATE_INT) !== false ) {
			// Update pema_task with values
			// Only the taskStart, taskEnd and orderid is updated
			$updated = $wpdb->update(
				'pema_task',
				array(
					'taskOrder' => $orderid,
					'taskStart' => $start,
					'taskEnd' => $end
				),
				array('taskID' => $taskID ),
				array(
					'%d',
					'%s',
					'%s'
				),
				array('%d')
			);
			// IF error when updating, die with message 'Error'
			if ( $updated === false ) {
				wp_die('Error');
			}
			// Else, i.e., new task is to be inserted
		} else {
			// If templateid is not an int... means someone has messed with it.. (inspect element)
			if ( filter_var($templateid, FILTER_VALIDATE_INT) === false ) {
				wp_die('Error');
			}
			// If orderid is not an int
			if ( filter_var($orderid, FILTER_VALIDATE_INT) === false ) {
				wp_die('Error');
			}
			
			// Get task template
			$task_template = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_task_template WHERE templateID = %d", $templateid));
			// If task template doesn't exist, die with message
			if ( $task_template === false ) {
				wp_die('Error');
			}

			// Get info about the answer with name from AJAX
			// We accept if it returns NULL, so dont need to check
			$getAnswer = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_answer_type WHERE answerID = '%d'", $task_template->answerType));

			if ( $getAnswer->typeName === 'Flag' ) {
				$hash = bin2hex(random_bytes(22));
			}

			// Insert into pema_task with variables
			$check = $wpdb->insert(
				'pema_task',
				array(
					'labID' => $labID,
					'answerID' => $task_template->answerType,
					'topicID' => $task_template->topic,
					'taskOrder' => $orderid,
					'taskName' => $task_template->templateName,
					'taskDescription' => $task_template->templateDescription,
					'taskStart' => $start,
					'taskEnd' => $end,
					'taskHash' => $hash
				),
				// Array used for prepeared statements, %d is int and %s is string
				array(
					'%d',
					'%d',
					'%d',
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s'
				)
			);
			// If error when inserting, die with error message 'Error'
			if ( $check === false ) {
				wp_die('Error');
			}
			// Get the newly inserted ID
			$taskID = $wpdb->insert_id;

			// Get all the hints for the templateID that was used
			$hints = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_hint_helper_template WHERE templateID = %d", $templateid));
			// For all hints
			foreach ( $hints as $hint ) {
				// Update pema_hint
				// Sets the taskID
				// This basically means that the hint has been "migrated" to a task instead of template_task
				$updated = $wpdb->insert(
					'pema_hint_helper_task',
					array(
						'hintID' => $hint->hintID,
						'taskID' => $taskID,
					),
					array(
						'%d',
						'%d'
					)
				);
				// If there was an error updating, die with message
				if ( $updated === false ) {
					wp_die('Error');
				}
			}

			// Get the parent ID
			$parent = $wpdb->get_row($wpdb->prepare("SELECT * FROM wp_posts WHERE ID IN (SELECT post_id FROM wp_postmeta WHERE meta_key = 'labID' AND meta_value = %d)", $labID));
			// If it couldn't find any, die with error message
			if ( $parent === NULL ) {
				wp_die('Error');
			}

			// Prepare array for insert into as new page
			$newPage = array(
				// Set post title to the same as task title
				'post_title' => $task_template->templateName,
				// Set content to the same as description
				'post_content' => $task_template->templateDescription,
				// Set status to publish
				'post_status' => 'publish',
				// Set type to page
				'post_type' => 'page',
				// Set author to the user who created the page
				'post_author' => $id,
				// Set the post parent to be the page for the lab the task is inserted under
				'post_parent' => $parent->ID
			);

			// Insert the page
			$get_page_id = wp_insert_post($newPage);
			// Update post meta, sets a template page for the task
			update_post_meta($get_page_id, '_wp_page_template', 'template-task.php');
			// Update post meta, set taskID to the ID that was inserted
			update_post_meta($get_page_id, 'taskID', $taskID);

			// Custom hook
			do_action('pema_after_insert_task', $taskID, $get_page_id);

		}
	} else {
		// Delete all foreign constraints first

		// Get all hints that can be deleted aswell
		// Checks if the hint is used in pema_hint_helper_template
		// Then checks if the hint is used more than once in pema_hint_helper_task
		$notexist = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_task WHERE taskID = %d AND hintID NOT IN (SELECT hintID FROM pema_hint_helper_template)) AND hintID IN (SELECT hintID FROM pema_hint_helper_task GROUP BY hintID HAVING COUNT(*) = 1)", $taskID));

		// Delete from pema_hint_helper_task where taskID is the taskID sent from AJAX
		$delete = $wpdb->delete('pema_hint_helper_task', array('taskID' => $taskID), array('%d'));
		// If there was an error deleting, die with message
		if ( $delete === false ) {
			wp_die('Error');
		}
		// It a hint can be deleted (i.e., not used by any task_templates or by another task)
		if ( $notexist ) {
			// Go through all hints and delete them
			foreach ( $notexist as $c ) {
				$deleted = $wpdb->delete('pema_hint', array('hintID' => $c->hintID), array('%d'));
				// If there was an error deleting, exit with error message
				if ( $deleted === false ) {
					wp_die('Error');
				}
			}
		}

		// Delete from pema_task_complete where taskID is taskID sent from AJAX
		$delete = $wpdb->delete('pema_task_complete', array('taskID' => $taskID), array('%d'));
		// If there was an error deleting, die with message
		if ( $delete === false ) {
			wp_die('Error');
		}

		// Get all pages for the taskID
		$posts = $wpdb->get_results($wpdb->prepare("SELECT * FROM wp_posts WHERE ID IN (SELECT post_id FROM wp_postmeta WHERE meta_key = 'taskID' AND meta_value = %d)", $taskID));
		// Go through all pages and delete them
		foreach ( $posts as $post ) {
			// True means delete instead of trash
			wp_delete_post($post->ID, true);
		}

		// Custom hook
		do_action('pema_before_task_delete', $taskID);

		// Delete the actual task
		$deleted = $wpdb->delete('pema_task', array('taskID' => $taskID), array('%d'));
		// If there was an error, die with message
		if ( $deleted === false ) {
			wp_die('Error');
		}
	}
	die();
}
/* ------------ Edits(overwrites) a task template ----------- */

// Adding action for function.
add_action('wp_ajax_pema_edit_task_template', 'pema_edit_task_template');

function pema_edit_task_template() {

	// Checks the nonce.
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initializing database connection.
	global $wpdb;

	// Get values from AJAX
	$id = 			$_POST["id"];
	$title =		$_POST["title"];
	$description =	$_POST["description"];
	$topic =		$_POST["topic"];
	$useAnswer = 	$_POST["useAnswer"];
	$answerType = 	$_POST["answerType"];

	// If useAnswer is true... (If an answer is to be used...)
	if ( $useAnswer === "true" ) {
		// Updating the database..
		$updated = $wpdb->update(
			'pema_task_template',
			array(
				'templateName' => $title,
				'templateDescription' => $description,
				'topic' => $topic,
				'answerType' => $answerType,
			),
			array( 'templateID' => $id ),
			array(
				'%s',
				'%s',
				'%d',
				'%s'
			)
		);
		// If error updating, die with message
		if ( $updated === false ) {
			wp_die('Error');
		}
	} else {
		// Updating the database..
		$updated = $wpdb->update(
			'pema_task_template',
			array(
				'templateName' => $title,
				'templateDescription' => $description,
				'topic' => $topic,
			),
			array( 'templateID' => $id ),
			array(
				'%s',
				'%s',
				'%d'
			)
		);
		// If error updating, die with message
		if ( $updated === false ) {
			wp_die('Error');
		}
	}

	// Custom hook
	do_action('pema_after_update_task_template', $id);

	die();
}

/* ----------------- Deletes a task template ---------------- */

// Adding action for function...
add_action('wp_ajax_pema_delete_task_template', 'pema_delete_task_template');

function pema_delete_task_template() {

	// Checks the nonce.
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize database connection.
	global $wpdb;

	// Gets ID.
	$idcol = $_POST["idcol"];

	// Check if it can delete hints from pema_hint
	// As it might be used in pema_hint_helper_task
	$hints = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_template WHERE templateID = %d AND hintID NOT IN (SELECT hintID FROM pema_hint_helper_task))", $idcol));

	// Deletes from pema_hint_helper_template
	$deleted = $wpdb->delete('pema_hint_helper_template', array('templateID' => $idcol), array('%d'));
	// If there was an error deleting, die with message
	if ( $deleted === false ) {
		wp_die('Error');
	}

	// If it can delete from pema_hint
	if ( $hints ) {
		// Go through hints
		foreach ( $hints as $hint ) {
			// Delete from DB
			$deleted = $wpdb->delete('pema_hint', array('hintID' => $hint->hintID), array('%d'));
			// If error deleting, die with message
			if ( $deleted === false ) {
				wp_die('Error');
			}
		}
	}

	// Custom hook
	do_action('pema_before_template_delete', $idcol);

	// Delete the task template
	$deleted = $wpdb->delete(
		'pema_task_template',
		array(
			'templateID' => $idcol
		),
		array('%d')
	);
	// If there was an error deleting, die with message
	if ( $deleted === false ) {
		wp_die('Error');
	}

	die();
}

/* -------------- Topic management -------------- */
// DELETE TOPIC
// Adding action for function...
add_action('wp_ajax_pema_delete_topic', 'pema_delete_topic');

function pema_delete_topic() {

	// Checks the nonce.
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize database connection.
	global $wpdb;

	// Gets ID.
	$idcol = $_POST["idcol"];

	// Check for foreign key constraint, i.e., if it is used by a task
	$foreign = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_task WHERE topicID = %d", $idcol));

	// If it is used, die with message
	if ( $foreign ) {
		echo "FK";
		die();
	}

	// Custom hook
	do_action('pema_before_topic_delete', $idcol);

	// Delete from DB
	$deleted = $wpdb->delete(
		'pema_topic',
		array(
			'topicID' => $idcol
		),
		array('%d')
	);
	// If error deleting, die
	if ( $deleted === false ) {
		die();
	}
	echo "Success";
	die();
}

// UPDATE TOPIC
// Adding action for function...
add_action('wp_ajax_pema_update_topic', 'pema_update_topic');

function pema_update_topic() {
	// Checks the nonce.
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initializing database connection.
	global $wpdb;

	// topicID and topicname
	$id = 	$_POST['id'];
	$name = $_POST['topicName'];

	// Update the database..
	$updated = $wpdb->update(
		'pema_topic',
		array(
			'topicName' => $name,
		),
		array( 'topicID' => $id),
		array(
			'%s'
		)
	);
	// If error updating, die with message
	if ( $updated === false ) {
		wp_die('Error');
	}

	do_action('pema_after_topic_update', $id);

	die();
}

// CREATE TOPIC
// Adding action for function.
add_action('wp_ajax_pema_create_topic', 'pema_create_topic');

function pema_create_topic() {
	// Checks the nonce. (Auto-kill on failure)
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize database connection.
	global $wpdb;

	// Getting data from AJAX.
	$name = $_POST["topicName"];

	// Insert into database..
	$inserted = $wpdb->insert(
		'pema_topic',
		array(
			'topicName' => $name
		),
		array(
			'%s'
		)
	);
	// If error inserting, die with message
	if ( $inserted === false ) {
		wp_die('Error');
	}

	// Get newly inserted ID
	$topicID = $wpdb->insert_id;

	// Custom hook
	do_action('pema_after_insert_topic', $topicID);

	echo $topicID;
	die();
}


/* -------------- UPDATING LAB OR GROUP -------------- */

// Make the function executable for logged in users
add_action('wp_ajax_pema_update', 'pema_update');

// Function for updating information about lab OR group
function pema_update() {
	// Check that the nonce is correct
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize DB connection
	global $wpdb;

	// Get all values from ajax
	$title = $_POST["title"];
	$desc = $_POST["desc"];
	// Lab ID or group ID
	$id = $_POST["id"];
	// Get all selected groups/users, i.e., groups/users that are to be added to the lab/group
	$addArr = $_POST["addarr"];
	// Get all non-selected groups/users, i.e., groups/users that should be removed from lab/group
	$removeArr = $_POST["removearr"];
	// Get the table name to update, either pema_user_has_group or pema_group_has_lab
	$updateTable = $_POST["updatetable"];
	// Get the first insertID, either labID or groupID
	$insertID1 = $_POST["insertid1"];
	// Get the second insertID, either groupID or userID
	$insertID2 = $_POST["insertid2"];

	// If updating a lab
	if ( $updateTable === 'pema_group_has_lab' ) {
		// Update to DB
		$updated = $wpdb->update(
			'pema_lab',
			array(
				'labName' => $title,
				'labDescription' => $desc
			),
			array('labID' => $id),
			array(
				'%s',
				'%s'
			),
			array('%d')
		);

		// If error updating, die with message
		if ( $updated === false ) {
			wp_die('Error');
		}

		// Custom hook
		do_action('pema_after_lab_update', $id);

	// If updating a group
	} elseif ( $updateTable === 'pema_user_has_group' ) {
		// Get the proficiency here, as lab doesn't send the info
		$proficiency = $_POST["proficiency"];
		// Update to DB
		$updated = $wpdb->update(
			'pema_group',
			array(
				'proficiencyID' => $proficiency,
				'groupName' => $title,
				'groupDescription' => $desc
			),
			array('groupID' => $id),
			array(
				'%d',
				'%s',
				'%s'
			),
			array('%d')
		);
		// If error updating, die with message
		if ( $updated === false ) {
			wp_die('Error');
		}

		// Custom hook
		do_action('pema_after_group_update', $id);
	}

	//If groups or users are to be added
	if ( $addArr ) {
		// Foreach group or user
		foreach ( $addArr as $add ) {
			// Check if the group or user already exist in Database
			$exist = $wpdb->get_row($wpdb->prepare("SELECT * FROM $updateTable WHERE $insertID2 = '%d' AND $insertID1 = '%d'", $add, $id));
			// If the group or user doesn't exist in the lab/group
			if ( !$exist ) {
				// Insert into Database
				$inserted = $wpdb->insert(
					// Table name
					$updateTable,
					array(
						// Insert into column $insertID2 with value $add
						// Will either be groupID or userID for column
						// And the group or user will be the value
						$insertID2 => $add,
						// Insert into column $insertID1 with value $id
						// Will either be labID or groupID for column
						// And the lab or group will be the value
						$insertID1 => $id
					),
					// Array for prepare statement
					// Will always be INT
					array(
						'%d',
						'%d'
					)
				);
				// If error inserting, die with message
				if ( $inserted === false ) {
					wp_die('Error');
				}
			}
		}
	}

	// If the groups or users are to be removed
	if ( $removeArr ) {
		// Foreach in array
		foreach ( $removeArr as $remove ) {
			// See if user/group exist in the group/lab
			$exist = $wpdb->get_row($wpdb->prepare("SELECT * FROM $updateTable WHERE $insertID2 = %d AND $insertID1 = %d", $remove, $id));
			// If it exist
			if ( $exist ) {
				//Delete it from DB
				$inserted = $wpdb->delete(
					// Table name
					$updateTable,
					array(
						// Delete row where $insertID2 = $remove
						$insertID2 => $remove,
						// Delete row where $insertID1 = $id
						$insertID1 => $id
					),
					// Array used for prepared statement
					// Will always be INT
					array(
						'%d',
						'%d'
					)
				);
				// If error deleting, die with message
				if ( $inserted === false ) {
					wp_die('Error');
				}
			}
		}
	}

	die();

}

/* -------------- DISPLAYING HINT FOR USERS -------------- */

add_action('wp_ajax_pema_show_hint', 'pema_show_hint');

// Function for displaying hints
function pema_show_hint() {

	// Check nonce
	check_ajax_referer('pema_ajax_nonce', 'security');

	// Initialize DB
	global $wpdb;

	// Get values from AJAX
	$taskID = $_POST["taskid"];
	$userid = $_POST["userid"];

	// Get the roles of the user
	$user_meta = get_userdata($userid);
	$user_roles = $user_meta->roles;

	// If user has role administrator or instructor
	if ( in_array('administrator', $user_roles) || in_array('instructor', $user_roles) ) {
		// Get all hints
		$adminhints = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_task WHERE taskID = %d)", $taskID));
		if ( $adminhints ) {
			// Go through all hints, and display them using echo
			foreach ( $adminhints as $adminhint ) {
				$proficiency = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_proficiency WHERE proficiencyID = %d", $adminhint->proficiencyID));
				 echo '<br><b>' . $proficiency->proficiencyName . ' hint:</b><br> ' . $adminhint->hintDescription . '<br>';
			}
		}

	// Else, means student requested show hint
	} else {

		// Get the hint for the user, might be experienced hint or beginner hint
		$hint = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_task WHERE taskID = %d) AND proficiencyID IN (SELECT proficiencyID FROM pema_group WHERE groupID IN (SELECT groupID FROM pema_user_has_group WHERE userID = %d))", $taskID, $userid));
		// If hint exist for task
		if ( $hint ) {
			// Display the hint
			echo '<br><b>Hint:</b><br> ' . $hint->hintDescription . '<br>';
		}
	}
	echo '<br>';

	// Custom hook
	do_action('pema_after_show_hint', $taskID, $userID);


	die();
}

/* -------------- FUNCTION FOR HOOKING WITH OPENSTACK PLUGIN -------------- */

add_action('wp_ajax_pema_openstack', 'pema_openstack');

// Function for running the openstack plugin
function pema_openstack() {

	check_ajax_referer('pema_ajax_nonce', 'security');

	$groups = $_POST["groups"];
	$labID = $_POST["labid"];

	// Do action,
	do_action('pema_adding_vms_to_groups', $groups, $labID);

	die();
}
?>
