<?php

/*


	============================
		STARTUP FUNCTIONS
	============================
*/

// Runs the function pema_add_post_startup when the theme is activated
add_action("after_switch_theme", "pema_add_startup");


// This function creates all relevant pages and roles (with capabilites)
function pema_add_startup() {

	/* ----------- CREATE ROLES/CAPABILITES ----------- */

	// Adding student role.
	add_role(
		// Role name.
		'student',
		// Display name.
		'Student'
		);

	// Adding instructor role.
	add_role(
		// Role name.
		'instructor',
		// Display name.
		'Instructor',
		// Array of capabilities.
		array(
			// Can access parts of the admin dashboard. (Native WP)
			'read' => true,
			// Can list users.
			'list_users' => true,
			// Can create users.
			'create_users' => true,
			// Can set user roles.
			'promote_users' => true,
			// Can delete users.
			'delete_users' => true,
			// Can remove users. (Because delete users is not enough apparently)).
			'remove_users' => true,
			// Can create a newsfeed-post.
			'pema_create_post' => true,
			// Can create lab.
			'pema_create_lab' => true,
			// Can edit labs.
			'pema_edit_lab' => true,
			// Can view labs.
			'pema_view_lab' => true,
			// Can manage tasks.
			'pema_manage_task' => true,
			// Can create tasks.
			'pema_create_task' => true,
			// Can create groups.
			'pema_create_group' => true,
			// Can manage topics.
			'pema_manage_topic' => true,
			// Can manage tasks for lab
			'pema_manage_tasks_lab' => true,
			// Can edit groups
			'pema_edit_group' => true,
			// Can see whos submitted tasks
			'pema_submitted_task' => true,
			// Can deactivate/activate plugins
			'activate_plugins' => true
		));
	// Getting admin and Adding administrator capabilities.
	$role = get_role('administrator');
	// Can create posts.
	$role->add_cap('pema_create_post');
	// Can create labs.
	$role->add_cap('pema_create_lab');
	// Can edit labs.
	$role->add_cap('pema_edit_lab');
	// Can view labs.
	$role->add_cap('pema_view_lab');
	// Can create groups.
	$role->add_cap('pema_create_group');
	// Can manage tasks.
	$role->add_cap('pema_manage_task');
	// Can create tasks.
	$role->add_cap('pema_create_task');
	// Can manage topics.
	$role->add_cap('pema_manage_topic');
	// Can manage tasks for lab
	$role->add_cap('pema_manage_tasks_lab');
	// Can edit groups
	$role->add_cap('pema_edit_group');
	// Can see whos submitted tasks
	$role->add_cap('pema_submitted_task');


	/* --- Removes unused default roles --- */

	// Gets array of wordpress roles.
	global $wp_roles;

	// Creates an array of roles to remove.
    $roles_to_remove = array(
		'subscriber',
		'contributor',
		'author',
		'editor'
	);

    // For every element in array.
    foreach ( $roles_to_remove as $role ) {
    	// If role exists.
        if ( isset($wp_roles->roles[$role]) ) {
        	// Remove role.
            $wp_roles->remove_role($role);
        }
	}

	/* ----------- ADD METADATA TO ADMIN USER ----------- */

	add_user_meta('1', 'pema-hide', 'show', true);


	/* ----------- CREATE DATABASE TABLES ----------- */

	// Creates all the tables
	pema_create_tables();

	/*-----------INSERT SOME TOPICS TO DATABASE ----------- */

	global $wpdb;			//Initialize Database
	// Sets a list of topics
	$topics = array('SQL Injection', 'XSS', 'Brute Force');
	// For each topic
	foreach ( $topics as $topic ) {
		// If topic does not already exist
		if ( !$wpdb->get_var("SELECT COUNT(*) FROM pema_topic WHERE topicName = '$topic'") ) {
			// Insert
			$wpdb->insert(
				// Table name
				'pema_topic',
				array(
					// Sets topicName to topic
					'topicName' => $topic
				)
			);
		}
	}

	// Defines the list of possible answer types a task can have.
	$answerTypes = array('Flag', 'String');
	// For every entry in the string.
	foreach ( $answerTypes as $answerType ) {
		// Verify it does not exist.
		if ( !$wpdb->get_var("SELECT COUNT(*) FROM pema_answer_type WHERE typeName = '$answerType'") ) {
		// Insert into..
		$wpdb->insert(
			 // table..
			'pema_answer_type',
			array(
				 // variable
				'typeName' => $answerType
				)
			);
		}
	}

	// Modify this array to add / remove proficiencies
	$proficiencys = array('Experienced', 'Beginner');
	// Adds proficiencies in the proficiency table when theme is activated
	foreach ( $proficiencys as $proficiency ) {
		// Check if it already exist
		if ( !$wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM pema_proficiency WHERE proficiencyName = '%s'", $proficiency)) ) {
			$wpdb->insert(
				'pema_proficiency',
				array(
					'proficiencyName' => $proficiency
				),
				array(
					'%s'
				)
			);
		}
	}
}




/* ------------- CREATE DATABASE TABLES ------------- */

function pema_create_tables() {
	// Use the global wpdb class, https://codex.wordpress.org/Class_Reference/wpdb
	global $wpdb;

	// Gets the database charset
	$charset_collate = $wpdb->get_charset_collate();

	//FIELD TYPES MUST BE LOWERCASE
	//TWO SPACES BETWEEN THE WORDS PRIMARY KEY AND THE DEFINITION OF IT
	// https://codex.wordpress.org/Creating_Tables_with_Plugins

	// Initializes pema_group table
	$pema_sql_group = 				"CREATE TABLE IF NOT EXISTS pema_group(
										groupID int(11) NOT NULL AUTO_INCREMENT,
										proficiencyID int(11),
										groupName varchar(60) NOT NULL,
										groupDescription text,
										FOREIGN KEY (proficiencyID) REFERENCES pema_proficiency(proficiencyID),
										PRIMARY KEY  (groupID)
									) $charset_collate;";

	// Initializes pema_user_has_group table
	$pema_sql_user_has_group = 		"CREATE TABLE IF NOT EXISTS pema_user_has_group (
										userID bigint(20) UNSIGNED,
										groupID int(11),
										FOREIGN KEY (userID) REFERENCES wp_users(ID),
										FOREIGN KEY (groupID) REFERENCES pema_group(groupID)
									) $charset_collate;";

	// Initializes pema_lab_template table
	$pema_sql_lab_template = 		"CREATE TABLE IF NOT EXISTS pema_lab_template (
										templateID int(11) NOT NULL AUTO_INCREMENT,
										templateName varchar(60),
										templateDescription text,
										templateCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										PRIMARY KEY  (templateID)
									) $charset_collate;";

	// Initializes pema_task table
	$pema_sql_task = 				"CREATE TABLE IF NOT EXISTS pema_task (
										taskID int(11) NOT NULL AUTO_INCREMENT,
										labID int(11),
										topicID int(11),
										taskOrder int(11),
										taskName varchar(60) NOT NULL,
										taskDescription text,
										answerID int(11),
										taskCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										taskStart timestamp NULL DEFAULT NULL,
										taskEnd timestamp NULL DEFAULT NULL,
										taskHash varchar(60),
										PRIMARY KEY  (taskID),
										FOREIGN KEY (topicID) REFERENCES pema_topic(topicID),
										FOREIGN KEY (answerID) REFERENCES pema_answer_type(answerID),
										FOREIGN KEY (labID) REFERENCES pema_lab(labID)
									) $charset_collate;";

	// initializes pema_task_template table
	$pema_sql_task_template = 		"CREATE TABLE IF NOT EXISTS pema_task_template (
										templateID int(11) NOT NULL AUTO_INCREMENT,
										templateName varchar(60),
										templateDescription text,
										topic int(11),
										templateCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										answerType int(11) NULL DEFAULT NULL,
										PRIMARY KEY  (templateID),
										FOREIGN KEY (topic) REFERENCES pema_topic(topicID),
										FOREIGN KEY (answerType) REFERENCES pema_answer_type(answerID)
									) $charset_collate;";

	$pema_sql_topic =				"CREATE TABLE IF NOT EXISTS pema_topic (
										topicID int(11) NOT NULL AUTO_INCREMENT,
										topicName varchar(60),
										PRIMARY KEY  (topicID)
									) $charset_collate;";

	$pema_sql_hint = 				"CREATE TABLE IF NOT EXISTS pema_hint (
										hintID int(11) NOT NULL AUTO_INCREMENT,
										proficiencyID int(11),
										hintDescription text,
										PRIMARY KEY  (hintID),
										FOREIGN KEY (proficiencyID) REFERENCES pema_proficiency(proficiencyID)
									) $charset_collate;";								//initializes pema_hint table
	// Initializes pema_lab table
	$pema_sql_lab = 				"CREATE TABLE IF NOT EXISTS pema_lab (
										labID int(11) NOT NULL AUTO_INCREMENT,
										labName varchar(60),
										labDescription text,
										labStart timestamp,
										labEnd timestamp,
							 			labCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										PRIMARY KEY  (labID)
									) $charset_collate;";

	// initializes pema_vm table
	$pema_sql_vm = 					"CREATE TABLE IF NOT EXISTS pema_vm (
										vmID int(11) NOT NULL AUTO_INCREMENT,
										vmCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										vmServerID varchar(100),
										vmIP varchar(15),
										vmKey text,
										PRIMARY KEY  (vmID)
									) $charset_collate;";


	$pema_sql_group_has_lab = 		"CREATE TABLE IF NOT EXISTS pema_group_has_lab (
										groupID int(11) NOT NULL,
										labID int(11) NOT NULL,
										FOREIGN KEY (groupID) REFERENCES pema_group(groupID),
										FOREIGN KEY (labID) REFERENCES pema_lab(labID)
									) $charset_collate;";

	$pema_sql_group_VM = 			"CREATE TABLE IF NOT EXISTS pema_group_vm (
										vmID int(11) NOT NULL,
										groupID int(11) NOT NULL,
										labID int(11) NOT NULL,
										FOREIGN KEY (vmID) REFERENCES pema_vm(vmID),
										FOREIGN KEY (groupID) REFERENCES pema_group(groupID),
										FOREIGN KEY (labID) REFERENCES pema_lab(labID)
									) $charset_collate;";

	$pema_sql_answer_type = 		"CREATE TABLE IF NOT EXISTS pema_answer_type (
		 								answerID int(11) NOT NULL AUTO_INCREMENT,
										typeName varchar(40),
										PRIMARY KEY  (answerID)
	 								) $charset_collate;";

	$pema_sql_task_complete =		"CREATE TABLE IF NOT EXISTS pema_task_complete (
										taskID int(11) NOT NULL,
										groupID  int(11) NOT NULL,
										submitted timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
										FOREIGN KEY (taskID) REFERENCES pema_task(taskID),
										FOREIGN KEY (groupID) REFERENCES pema_group(groupID)
									) $charset_collate;";

	$pema_sql_proficiency =			"CREATE TABLE IF NOT EXISTS pema_proficiency (
										proficiencyID int(11) NOT NULL AUTO_INCREMENT,
										proficiencyName varchar(60) NOT NULL,
										PRIMARY KEY  (proficiencyID)
									) $charset_collate;";

	$pema_sql_hint_helper_task = 	"CREATE TABLE IF NOT EXISTS pema_hint_helper_task (
										hintID int(11),
										taskID int(11),
										FOREIGN KEY (hintID) REFERENCES pema_hint(hintID),
										FOREIGN KEY (taskID) REFERENCES pema_task(taskID)
									) $charset_collate;";

	$pema_sql_hint_helper_template = "CREATE TABLE IF NOT EXISTS pema_hint_helper_template (
										hintID int(11),
										templateID int(11),
										FOREIGN KEY (hintID) REFERENCES pema_hint(hintID),
										FOREIGN KEY (templateID) REFERENCES pema_task_template(templateID)
									) $charset_collate;";


	// includes upgrade.php which is used to create/update DB tables
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta($pema_sql_proficiency);
	dbDelta($pema_sql_topic);				// Adds pema_topic table
	dbDelta($pema_sql_group);				// Adds pema_group table
	dbDelta($pema_sql_user_has_group);		// Adds pema_user_has_group table
	dbDelta($pema_sql_lab_template);		// Adds pema_lab_template table
	dbDelta($pema_sql_lab);					// Adds pema_lab table
	dbDelta($pema_sql_answer_type);			// Adds pema_answer_type table
	dbDelta($pema_sql_task);				// Adds pema_task table
	dbDelta($pema_sql_task_template);		// Adds pema_task_template table
	dbDelta($pema_sql_hint);				// Adds pema_hint table
	dbDelta($pema_sql_vm);					// Adds pema_vm table
	dbDelta($pema_sql_group_has_lab);		// Adds pema_group_has_lab table
	dbDelta($pema_sql_group_VM);			// Adds pema_group_vm table
	dbDelta($pema_sql_task_complete);		// Adds pema_task_complete table
	dbDelta($pema_sql_hint_helper_task);
	dbDelta($pema_sql_hint_helper_template);

	/* TO DROP ALL ADDED TABLES RUN
	   Please keep order and key constraints in mind when modifying ))

	DROP TABLE pema_hint_helper_template;
	DROP TABLE pema_hint_helper_task;
	DROP TABLE pema_task_complete;
	DROP TABLE pema_hint;
	DROP TABLE pema_task_template;
	DROP TABLE pema_task;
	DROP TABLE pema_answer_type;
	DROP TABLE pema_user_has_group;
	DROP TABLE pema_group_has_lab;
	DROP TABLE pema_group_vm;
	DROP TABLE pema_vm;
	DROP TABLE pema_lab;
	DROP TABLE pema_group;
	DROP TABLE pema_lab_template;
	DROP TABLE pema_topic;
	DROP TABLE pema_proficiency;
	*/
}
?>
