jQuery(document).ready(function($){
	// Activate plugin on element
	$('#pema-create-lab-add-group-select').chosen();

	// When radio button is changed
	$('input:radio[name="pema-create-lab-labtemplate-radioname"]').change(function() {
		// If value is yes (e.g., true)
		if ( $(this).val() === 'yes' ) {
			//alert("Yes is checked");
			// Show the template select option
			$('#pema-create-lab-labtemplate-select').show();
			// Set title input field to readonly, e.g., user cannot input anything
			$('#pema-create-lab-title-input').prop('readonly', true);
			// Set description input field to readonly, e.g., user cannot input anything
			$('#pema-create-lab-description-textarea').prop('readonly', true);
		// If button is set to "no"
		} else {
			// Hide the template select option
			$('#pema-create-lab-labtemplate-select').hide();
			// Disable readonly for title input
			$('#pema-create-lab-title-input').prop('readonly', false);
			// Disable readonly for description input
			$('#pema-create-lab-description-textarea').prop('readonly', false);
		}
	});

	// When template select is selected
	$('[name="pema-template-select"]').change(function() {

		// Get the description data
		var description = $(this).find('option:selected').data('description');

		// Set value for title input field
		$('#pema-create-lab-title-input').val($(this).val());
		// Set value for description input field
		$('#pema-create-lab-description-textarea').val(description);
	});

	$('#pema-create-lab-startdate-input').flatpickr({
		// Enable time (e.g., 12:00)
		enableTime: true,
		// Date format
		dateFormat: "Y-m-d H:i",
		// Sets 24-hour times
		time_24hr: true,
		weekNumbers: true,
		// Minimum date is today
		minDate: "today",
		// For the Locale..
		locale: {
			// Set first day of week to be monday, like it should be.
			firstDayOfWeek: 1
		},
		// Use plugin to select a range
		"plugins": [new rangePlugin({ input: "#pema-create-lab-enddate-input" })]
	});	

	// On submit
	$(document).on('submit', '#pema-create-lab-submit-form', function(e) {
		// Prevent the default action of the form to be triggered (e.g., how forms ALWAYS refreshes when submitted, annoying -_-)
		e.preventDefault();

		// Gets data from input fields
		var title = $('#pema-create-lab-title-input').val();
		var description = $('#pema-create-lab-description-textarea').val();
		var startdate = $('#pema-create-lab-startdate-input').val();
		var enddate = $('#pema-create-lab-enddate-input').val();
		var usetemplate = $('#pema-create-lab-labtemplate-true').is(":checked");
		var groups = $('#pema-create-lab-add-group-select').val();
		var check = false;
		var labid;

		// Make sure that all input fields are entered
		if ( title.trim() && description.trim() && startdate && enddate ) {
			$.ajax({
				// Gets ajax url
				url : ajax_params.ajaxurl,
				// Sets type to be post
				type : 'post',
				dataType : 'json',
				// Define what ajax will send to server
				data : {
					// Send title
					title : title,
					// Send description
					description : description,
					// Send startdate
					startdate : startdate,
					// Send enddate
					enddate : enddate,
					// Send usetemplate, e.g., if it should use a preexisting template or not
					usetemplate : usetemplate,
					// Send loggedinuserID, although this is not used YET
					id : ajax_params.loggedinuserID,
					// Gets groups
					groups : groups,
					// Gets the nounce
					security : ajax_params.ajax_nonce,
					// Run this function
					action : 'pema_deploy_lab'
				},
				// If error in ajax
				error : function ( response ) {
				},
				success : function ( data ) {
					// If text is equal to success
					if ( data['text'] === 'Success' ) {
						// Set bool so that next function can run
						check = true;
						// Set labID, will be used by next function
						labid = data['labid'];
						// Give message
						alertSuccess('New lab created!');
						// Reset fields/form
						$('#pema-create-lab-add-group-select option:selected').removeAttr('selected');
						$('#pema-create-lab-add-group-select option').trigger('chosen:updated');
						$('#pema-create-lab-submit-form')[0].reset();
					// If it already exist, give message
					} else if ( data['text'] === 'Exist' ) {
						alertError('A lab with that name already exists!');
					// Else, give error message
					} else {
						alertError('Something went wrong, please contact the admin!');
					}
				}
				// When the other ajax call is done, run a new function!
			}).done(function() {
				// If check is true, i.e., previous ajax call was "successful"
				if ( check ) {
					// ajax call for calling the function that runs openstack plugin
					$.ajax({
						url : ajax_params.ajaxurl,
						type : 'post',
						data : {
							labid : labid,
							groups : groups,
							security : ajax_params.ajax_nonce,
							action : 'pema_openstack'
						},
						error : function ( response ) {
						},
						success : function ( data ) {
						}
					})
				}
			});
		// If all input fields are not entered, give error
		} else {
			alertError('Please fill in all the fields.');
		}

	});

});