jQuery(document).ready(function($) {
	// Use jquery plugin on element
	$('#pema-edit-group-users-manage').multiSelect({
		selectableHeader: "<div id='pema-edit-group-users-select-selectable'>Selectable users</div>",
		selectionHeader: "<div id='pema-edit-group-users-select-selected'>Selected users</div>"
	});

	// On submit form
	$(document).on('submit', '#pema-edit-group-form', function(e){
		// Prevent action on form
		e.preventDefault();

		// Get url info
		var searchParams = new URLSearchParams(window.location.search);
		// If params in URL has labID 
		if ( searchParams.has('groupID') ) {

			// Get title
			var title = $('#pema-edit-group-name-input').val();
			// Get description
			var desc = $('#pema-edit-group-description-textarea').val();
			// Get param from URL
			var id = searchParams.get('groupID');

			// Array for storing all selected groups, i.e., groups that should be in the lab
			var selectedUsers = [];
			// For all selected
			$('#pema-edit-group-users-manage-select option:selected').each(function(){
				// Add the value to the array
				selectedUsers.push($(this).val());
			});
			// Array for storing all non-selected groups, i.e., groups that should not be in the lab
			var notselectedUsers = [];
			// For all non-selected
			$('#pema-edit-group-users-manage-select option').not(':selected').each(function() {
				// Add the value to the array
				notselectedUsers.push($(this).val());
			});
			var proficiency = $('input[name=pema-edit-group-proficiency-radio]:checked').val();

			if ( selectedUsers.length <= 3 ) {
				// If title and desc has content, i.e., the fields are not empty 
				if ( title.trim() && desc.trim() ) {
					// Ajax
					$.ajax({
						// Get ajax URL
						url : ajax_params.ajaxurl,
						// Get type
						type : 'post',
						// Data
						data : {
							// Send title
							title : title,
							// Send description
							desc : desc,
							// Send labID
							id : id,
							// Send what table to update
							updatetable : 'pema_user_has_group',
							// Send the first field to check against
							insertid1 : 'groupID',
							// Send the second field to check against
							insertid2 : 'userID',
							// Send all selected groups
							addarr : selectedUsers,
							// Send all non-selected groups
							removearr : notselectedUsers,
							proficiency : proficiency,
							// Send the nonce
							security: ajax_params.ajax_nonce,
							// Run this function
							action: 'pema_update'
						},
						// On error
						error : function( response ) {
						},
						// On success
						success : function(data) {
							if ( data !== 'Error' ) {
								// Run function with data
								alertSuccess('Group updated!');
							// Else, reset form
							} else {
								$('#pema-edit-group-form')[0].reset();
								alertError('Something went wrong, please contact the admin.');
							}
						}
					});
				// Else, give error
				} else {
					alertError('Please fill in the fields!', 'orange');
				}
			// Else, give error
			} else {
				alertError('Cannot be more than 3 users in a single group!', 'orange');
			}
			
		}
	});

	$(document).on('click', '#pema-edit-group-delete-button', function() {
		// Get url info
		var searchParams = new URLSearchParams(window.location.search);
		if ( searchParams.has('groupID') ) {
			var id = searchParams.get('groupID');
			// Displays a confirm-box using jquery-confirm.
			$.confirm({
				// Setting parameters.. title.
				title: 'Group deletion',
				// Content.
				content: 'Are you sure you want to delete this group?',
				// Style "type".
				type: 'red',
				// Adds a little colour animation.
				typeAnimated: true,
				closeAnimation: 'top',
				// Disables bootstrap (allows for customization)
				useBootstrap: false,
				// Sets the alert-box width.
				boxWidth: '30%',
				theme: 'Material',
				// Options:
				buttons: {
					// Yes.
					Yes: function() {
						// Calls ajax.
						$.ajax({
							// Gets ajax url.
							url: ajax_params.ajaxurl,
							// Sets type to post.
							type: 'post',
							// Data ..
							data: {
								// ID of the topic.
								id: id,
								// Nonce.
								security: ajax_params.ajax_nonce,
								// Run this function.
								action: 'pema_delete_group'
							},
							// If ajax throws an error.
							error: function( response ) {
								// Log error to console.
								console.log(response);
							},
							// On success.
							success: function( data ) {
								console.log(data);
								// Foreign key constraint
								if ( data === "FK" ) {
									alertError('This group is a part of an active lab and cannot be deleted!');
								// Elseif, error
								} else if ( data === "Error") {
									alertError('Something went wrong, please contact the admin.');
								// Else, successful
								} else {
									alertSuccess('Group deleted!');
									// Redirect Page
									window.location.href = data;
								}
							}
						});
					},
					// Cancel
					Cancel: function() {
						// Do nothing :)
					}
				}
			});
		}
	});
});