<?php

// Get header and sidebars
get_header();

get_sidebar('home');

get_sidebar('top');

// Initialize DB
global $wpdb;
// Get info about the page
global $post;


// Get relevant lab.
$lab = $wpdb->get_row("SELECT * FROM pema_lab WHERE labID = (SELECT meta_value FROM wp_postmeta WHERE meta_key = 'labID' AND post_id = '$post->ID')");
// Get tasks.
$tasks = $wpdb->get_results("SELECT * FROM pema_task WHERE labID = (SELECT meta_value FROM wp_postmeta WHERE meta_key = 'labID' AND post_id = '$post->ID')");

// Go through all tasks in this lab, and get their topics.
foreach ( $tasks as $task ) {
	$topics[] = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_topic WHERE topicID = %d",$task->topicID));
}

// If user is logged in
if ( is_user_logged_in() ) {
	?>
	<div class="pema-content">
		<div id="pema-display-lab-wrapper">
			<div id="pema-display-lab-title">
				<h1><?php echo esc_html($post->post_title); ?></h1>
			</div><br>

			<div id="pema-display-lab-startdate">
				<p>This lab starts: <b><?php echo esc_html($lab->labStart); ?></b></p>
			</div>

			<div id="pema-display-lab-enddate">
				<p>This lab ends:  <b><?php echo esc_html($lab->labEnd); ?></b></p>
			</div>

			<div id="pema-lab-time-remaining">
				<?php
				// Converts labStart and End to unix-time.
				$startTime = strtotime($lab->labStart);
				$endTime = strtotime($lab->labEnd);
				// Loads the contents of a lab if the lab is active (started and not ended), or the user is either admin or instructor.
 				if ( ( $startTime < time() && $endTime > time() ) || ( current_user_can('administrator') || current_user_can('instructor')) ) {
					// Find difference between current time() and lab end in seconds.
					$timeLeft = $endTime - time();

					// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
					$days = floor($timeLeft / 86400);
					$timeLeft %= 86400;

					$hours = floor($timeLeft / 3600);
					$timeLeft %= 3600;

					$minutes = floor($timeLeft / 60);
					$timeLeft %= 60;

					// The following checks are for instructors and admins.
					// Checks if lab has not started yet.
					if ( $startTime > time() ) {
					?>
						<p>Lab has not started yet! </p>
					<?php
					// Checks if lab has started, and has not finished. If so, return time remaining, else; do nothing.
					} elseif ( $minutes > 0 ) {
					?>
						<p>Time remaining: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
					<?php
					}
					?>
					<br>
					<div id="pema-display-lab-topics">
						<h2>Topics: </h2>
						<?php
						// Checks if there are topics presents.
						if ( !empty($topics) ){
							foreach ( $topics as $topic ) {
								?>
								<h3><?php echo esc_html($topic->topicName); ?> </h3> <?php
							}
						// If there are no topics, print message instead.
						} else {
							?> <h3> This lab has no topics! </h3> <?php
						}
						?>
						<br>
					</div>

					<br><br>

					<div id="pema-display-lab-description">
						<p><?php echo esc_html($post->post_content); ?></p>
					</div><br>

					<br><br>

					<?php
					// Custom hook
					do_action('pema_display_lab_view', $post->ID);
					?>

					<div id="pema-display-lab-tasks-wrapper">
					<?php
						// For all tasks, get information then create a clickable box and display some of this information.
						foreach ( $tasks as $task ) {
							$tid = $wpdb->get_row(
								"SELECT * FROM wp_posts WHERE post_status = 'publish'
								AND ID = (SELECT post_id FROM wp_postmeta WHERE meta_key = 'taskID'
								AND meta_value = (SELECT taskID FROM pema_task WHERE taskID = $task->taskID))"
								);
							?>
							<a href="<?php echo get_page_link($tid->ID);?>">
							<div id="pema-display-task-<?php echo esc_attr($task->taskID); ?>">
								<div id="pema-display-task-title">
					 			 	<h3><?php echo esc_html($task->taskName); ?></h3>
								</div><br>

								<?php
								if ( $task->taskStart ) { ?>
									<div id="pema-display-task-startdate">
										<p>Start:
											<b><?php echo esc_html($task->taskStart); ?></b></p>
									</div>

									<div id="pema-display-task-enddate">
										<p>Deadline:
											<b><?php echo esc_html($task->taskEnd); ?></b></p>
									</div>

									<div id="pema-task-time-left">
										<?php
										// Convert task start and end times to unix-timestamp.
										$startTime = strtotime($task->taskStart);
										$endTime = strtotime($task->taskEnd);
										// Check if task has not started yet. If correct, print time remaining until start.
										if ( $startTime > time() ) {
											// Find difference between task start and current time in seconds.
											$timeLeft = $startTime - time();

											// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
											$days = floor($timeLeft / 86400);
											$timeLeft %= 86400;

											$hours = floor($timeLeft / 3600);
											$timeLeft %= 3600;

											$minutes = floor($timeLeft / 60);
											$timeLeft %= 60;

											?>
											<p>Task starts in: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
											<?php
										// Check if task has not finished. If correct, print time remaining until end.
										} elseif ( $endTime > time() ) {
											// Find difference between task end and current time() in seconds.
											$timeLeft = $endTime - time();

											// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
											$days = floor($timeLeft / 86400);
											$timeLeft %= 86400;

											$hours = floor($timeLeft / 3600);
											$timeLeft %= 3600;

											$minutes = floor($timeLeft / 60);
											$timeLeft %= 60;

											?>
											<p>Time remaining: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
											<?php
										// If task has finished, print message saying so.
										} else { ?>
											<p>This task has ended. </p>
											<?php
										} ?>
									</div>

									<br>
									<?php
								} else {
									?>
									<div id="pema-display-task-notime">
									<p>This task does not have a time limit!</p>
									</div>
									<br>
									<?php
								} ?>
							</div>
							</a>
							<?php
						}
					?>
					</div>

					<?php
				// Check if lab has not started yet. If so, print time until it starts.
				} elseif ( $startTime >= time() ) {
					// Find difference between current time() and lab start in seconds.
					$timeLeft = $startTime - time();

					// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
					$days = floor($timeLeft / 86400);
					$timeLeft %= 86400;

					$hours = floor($timeLeft / 3600);
					$timeLeft %= 3600;

					$minutes = floor($timeLeft / 60);
					$timeLeft %= 60;

					?>
					<p>Lab starts in: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
					<?php
				// If lab has ended, print message saying so.
				} else {
					?>
					<p>This lab has ended. </p>
					<?php
				}
				?>
			</div>


		</div>
	</div>

	<?php

}
// Get footer
get_footer();




?>
