<?php
/*
Template Name: Task Page

*/
// Get header and sidebars
get_header();

get_sidebar('home');

get_sidebar('top');

// Initialize DB
global $wpdb;
// Get what post is being access
global $post;

//Gets current time in mysql format
$time = current_time('mysql');

// Gets the task.
$task = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_task WHERE taskID = (SELECT meta_value FROM wp_postmeta WHERE meta_key = 'taskID' AND post_id = %d)",$post->ID));

// Gets the tasks topic.
$topic = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_topic WHERE topicID = %d",$task->topicID));

 // Gets the lab of this task, to decide timestamps if the task does not have timestamps itself.
$lab = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_lab WHERE labID IN (SELECT meta_value FROM wp_postmeta WHERE meta_key = 'labID' AND post_id = %d )", $post->post_parent));

// Declaring taskActive, used to decide wether to display the contents of a task or not. Setting as default FALSE.
$taskActive = FALSE;

// If user is logged in
if ( is_user_logged_in() ) {
	?>
	<div class="pema-content">
		<div id="pema-display-task-wrapper">
			<div id="pema-display-task-title">
				<h1><?php echo esc_html($post->post_title); ?></h1>
			</div><br>

			<?php
			// Checking if the task has time constraints.
			if ( $task->taskStart ) { ?>
				<div id="pema-display-task-startdate">
					<p>Start:
						<b><?php echo esc_html($task->taskStart); ?></b></p>
				</div>

				<div id="pema-display-task-enddate">
					<p>Deadline:
						<b><?php echo esc_html($task->taskEnd); ?></b></p>
				</div>

				<div id="pema-display-task-time-left">
					<?php
					// Convert task start and end times to unix-timestamp.
					$startTime = strtotime($task->taskStart);
					$endTime = strtotime($task->taskEnd);
					// Check if task has not started yet. If correct, print time remaining until start.
					if ( $startTime > time() ) {
						// Find difference between task start and current time in seconds.
						$timeLeft = $startTime - time();

						// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
						$days = floor($timeLeft / 86400);
						$timeLeft %= 86400;

						$hours = floor($timeLeft / 3600);
						$timeLeft %= 3600;

						$minutes = floor($timeLeft / 60);
						$timeLeft %= 60;

						?>
						<p>Task starts in: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
						<?php
					// Check if task has not finished. If correct, print time remaining until end.
					} elseif ( $endTime > time() ) {
						// Find difference between task end and current time() in seconds.
						$timeLeft = $endTime - time();

						// Recalculate to days, hours and minutes left. (floor() rounds down to nearest integer)
						$days = floor($timeLeft / 86400);
						$timeLeft %= 86400;

						$hours = floor($timeLeft / 3600);
						$timeLeft %= 3600;

						$minutes = floor($timeLeft / 60);
						$timeLeft %= 60;

						?>
						<p>Time remaining: <b><?php echo $days . " days, " . $hours . " hours, " . $minutes . " minutes." ;  ?></b></p>
						<?php
						// Set taskActive to true.
						$taskActive = TRUE;
					// If task has finished, print message saying so.
					} else { ?>
						<p>This task has ended. </p>
						<?php
					}
					?>
				</div>

				<?php
			} else {
				?>
				<div id="pema-display-task-notime">
				<p>This task does not have a time limit!</p>
				</div>
				<?php
				// Cast the labs timestamps to unix-timestamp format.
				$startTime = strtotime($lab->labStart);
				$endTime = strtotime($lab->labEnd);
				// Check if the lab has started and not ended, or that the user is administrator or instructor. If correct, set value to display task contents.
				if ( ( $startTime < time() && $endTime > time() ) || ( current_user_can('administrator') || current_user_can('instructor')) ) {
					// Set taskActive as TRUE.
					$taskActive = TRUE;
				}
			}
			?>

			<br>
			<?php
			// Checks if the taskActive value is true. If it is, display the task contents.
			if ( $taskActive ) {
				// If loged in user has the role admin or instructor
				if ( current_user_can('administrator') || current_user_can('instructor') ) {
					// Check if any hints exist for this task
					$hint = $wpdb->get_results($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_task WHERE taskID = %d)", $task->taskID));
				} else {
					// Else check if there is a hint for the logged in user
					$hint = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_hint WHERE hintID IN (SELECT hintID FROM pema_hint_helper_task WHERE taskID = %d) AND proficiencyID IN (SELECT proficiencyID FROM pema_group WHERE groupID IN (SELECT groupID FROM pema_user_has_group WHERE userID = %d))", $task->taskID, get_current_user_id()));
				}
				?>
				<div id="pema-display-task-topic">
					<h2>Topic: <?php echo esc_html($topic->topicName); ?></h2>
				</div><br><br>

				<div id="pema-display-task-description">
					<p style="white-space: pre-line"><?php echo esc_html($post->post_content); ?></p>
				</div><br>

				<?php
				// Custom hook
				do_action('pema_display_task_view', $post->ID);
				?>

				<div id="pema-display-task-hint">
					<?php 
					// If there are hints for the user
					if ( $hint ) {
						?>
						<button id="pema-display-task-hint-button">Show hint</button>
						<?php
					} else {
						?>
						<p>This task does not have any hints.</p>
						<?php
					}
					?>
					
				</div><br>
				

				<div id="pema-display-task-input">
					<?php
					// Get currently logged in user ID
					$userid = get_current_user_id();
					// Get what answer type the task has
					$requireAnswer = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_answer_type WHERE answerID = %d", $task->answerID));
					// Check if the task has been completed
					$answered = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_task_complete WHERE groupID = (SELECT groupID FROM pema_user_has_group WHERE userID = %d) AND taskID = %d", $userid, $task->taskID));
					// If task completed, display green message saying its completed
					if ( $answered !== NULL ) {
						?>
						<p style="color:green">You have successfully completed this task at <?php echo esc_html($answered->submitted); ?>!</p>
						<?php
						// Else if, the task requires an answer, display input field
					} elseif ( $requireAnswer !== NULL ) {
						?>
						<div id="pema-display-task-input">
							<input id="pema-display-task-input-box" data-id="<?php echo esc_attr($task->taskID); ?>" type="text" placeholder="Answer hash">
						</div>
						<div id="pema-display-task-submit">
							<button id="pema-display-task-submit-button">Submit</button>
						</div>
						<?php
						// Else, display message that the task does not require an answer
					} else {
						?>
						<div id="pema-display-task-no-answer">
							<p>This task does not require an answer</p>
						</div>
						<?php
					}
					?>
				</div>
				<?php
			}
			?>
		</div><br><br>
		<div id="pema-display-task-next-task">
			<?php
			// Store the taskorder in a variable so we can use it
			$tempvar = $task->taskOrder;
			// Get the previous orderID
			$previousorderid = --$tempvar;
			// Gets the previous post
			$previoustask = $wpdb->get_row($wpdb->prepare("SELECT * FROM wp_posts WHERE post_status = 'publish' AND ID = (SELECT post_id FROM wp_postmeta WHERE meta_key = 'taskID' AND meta_value IN (SELECT taskID FROM pema_task WHERE taskOrder = %d AND labID = %d))", $previousorderid, $lab->labID));
			// If the previous task exist, display link to it
			if ( $previoustask !== NULL ) {
				?>
				<a id="pema-display-task-next-task-href" href="<?php echo get_page_link($previoustask->ID);?>">Previous task</a>
				<?php
			}
			// Store the taskorder in another variable so we can use it
			$tempvar2 = $task->taskOrder;
			// Get the next orderID
			$nextorderid = ++$tempvar2;
			// Gets the next post
			$nexttask = $wpdb->get_row($wpdb->prepare("SELECT * FROM wp_posts WHERE post_status = 'publish' AND ID = (SELECT post_id FROM wp_postmeta WHERE meta_key = 'taskID' AND meta_value IN (SELECT taskID FROM pema_task WHERE taskOrder = %d AND labID = %d))", $nextorderid, $lab->labID));
			// If the next post exist, display link to it
			if ( $nexttask !== NULL ) {
				?>
				<a id="pema-display-task-next-task-href" href="<?php echo get_page_link($nexttask->ID);?>">Next task</a>
				<?php
			}
			?>

		</div>
	</div>

	<?php

}
// Gets the footer
get_footer();



?>
