<?php
/**
* Plugin Name:  Openstack NTNU
* Description:  Plugin for using the Openstack Libary, used to deploy and restart openstack instances for students
* Author:   PEMA
*/

// Function for adding to the sidebar, adds link to the page that is created on plugin activation
function pema_plugin_openstack_init() {
    // Get all info about a page called "VM Management"
    $page = get_page_by_title('VM Management');
    // Create a href link to the page
    ?>
    <li><a href="<?php echo get_page_link($page->ID);?>">VM Management</a></li>
    <?php
}

// Hook into 'pema_after_sidebar_top' and run the function 'pema_plugin_openstack_init'
add_action('pema_after_sidebar_top', 'pema_plugin_openstack_init');

// Register and enqueue both jQuery script and css stylesheet
function pema_plugin_openstack_script() {
    wp_register_script('pema_api_openstack_script', plugins_url('openstack-api-script.js', __FILE__), array( 'jquery' ));
    wp_enqueue_script('pema_api_openstack_script');

    wp_register_style('pema_api_openstack_style', plugins_url('openstack-api-style.css', __FILE__));
    wp_enqueue_style('pema_api_openstack_style');
}
add_action('wp_enqueue_scripts', 'pema_plugin_openstack_script');


// Function for creating the header thats gonna be used to run a POST api query
function pema_plugin_openstack_api_arg_post($content) {
    global $wpdb;
    // Get the current token, then use the token in the $args
    $token = $wpdb->get_row("SELECT * FROM pema_openstack_token WHERE tokenID = 1");

    $args = array(
        "headers" => array(
            "Content-Type" => "application/json",
            "X-Auth-Token" => $token->tokenToken
        ),
        "timeout" => 60,
        "body" => $content
    );
    return $args;
}

// Function for creating the header thats gonna be used to run a GET api query
function pema_plugin_openstack_api_arg_get() {
    global $wpdb;
    // Get the current token, then use the token in the $args
    $token = $wpdb->get_row("SELECT * FROM pema_openstack_token WHERE tokenID = 1");

    $args = array(
        "headers" => array(
            "X-Auth-Token" => $token->tokenToken
        ),
        "timeout" => 60
    );
    return $args;
}


// Function for starting VMs for the groups
function pema_plugin_openstack_backend($groups, $labID) {

    if ( $groups ) {
        foreach( $groups as $group ) {
            // Initialize DB connection
            global $wpdb;

            // Get the group name
            $group_name = $wpdb->get_row($wpdb->prepare("SELECT * FROM pema_group WHERE groupID = %d", $group));

            // Replace spaces and _ with -
            $group_name->groupName = preg_replace("/[\s_]/", "-", $group_name->groupName);

            // Create a key name and server name, also set the project_id
            $key_name = 'key-' . $labID . $group . '-' . $group_name->groupName;
            $server_name = 'server-' . $labID . $group . '-' . $group_name->groupName;
            $project_id = '80ac64dc5af74e5fbd1724e0341256ce';

            // Run function with argument, used to create the header with token and data to send
            $args = pema_plugin_openstack_api_arg_post('{ "keypair": { "name": "' . $key_name . '"} }');

            // Run the api query to create a keypair
            $response = wp_remote_post('https://api.skyhigh.iik.ntnu.no:8774/v2.1/os-keypairs', $args);

            // If wordpress error, die with message
            if ( is_wp_error($response) ) {
                $error_message = $response->get_error_message();
                wp_die($error_message);
            }
            // Decode the response message
            $decoded = json_decode(wp_remote_retrieve_body($response));
            if ( isset($decoded->error) ) {
                wp_die('Error');
            }
            //wp_die(print_r($decoded));

           
            // Store the private and public key
            $keypair_priv = $decoded->keypair->private_key;
            //$keypair_pub = $decoded->keypair->public_key;


            // Set arguments for creating server in openstack
            $args = pema_plugin_openstack_api_arg_post('{ "server": { "name": "' . $server_name . '", "imageRef": "2d59ed0c-f5a9-4dee-bc80-bf994e620d3c", "flavorRef": "f9d74bd7-38be-4b90-9b09-1db9d8a83cf1", "key_name": "' . $key_name . '", "adminPass": "admin"}}');

            // Create server
            $response = wp_remote_post('https://api.skyhigh.iik.ntnu.no:8774/v2.1/servers', $args);

            // If error, die with message
            if ( is_wp_error($response) ) {
                $error_message = $response->get_error_message();
                wp_die($error_message);
            }

            // Decode the response
            $decoded = json_decode(wp_remote_retrieve_body($response));
            if ( isset($decoded->error) ) {
                wp_die('Error');
            }

            // Get the server_id of the newly created server
            $server_id = $decoded->server->id;

            // Sleep.. need to wait until the server has been started first
            sleep(7);

            // Set arguments for getting info, then get information about the newly created server
            $args = pema_plugin_openstack_api_arg_get();
            $response = wp_remote_get('https://api.skyhigh.iik.ntnu.no:8774/v2.1/servers/' . $server_id . '/os-interface', $args);

            // IF error in wordpress wp_remote_get
            if ( is_wp_error($response) ) {
                $error_message = $response->get_error_message();
                wp_die($error_message);
            }

            // Decode the response into json format
            $decoded = json_decode(wp_remote_retrieve_body($response));
            if ( isset($decoded->error) ) {
                wp_die('Error');
            }

            // Get the port ID of the server (where we hook the floating IP to)
            $port_id = $decoded->interfaceAttachments[0]->port_id;

            // Set args for creating a floating IP and associating it with a server (in this case our created server)
            $args = pema_plugin_openstack_api_arg_post('{ "floatingip": {"tenant_id": "' . $project_id . '", "project_id": "' . $project_id . '", "floating_network_id": "730cb16e-a460-4a87-8c73-50a2cb2293f9", "description": "IP for group number: ' . $group . '", "port_id": "' . $port_id . '"}}');

            // Run the query for creating a floating IP and associating it
            $response = wp_remote_post('https://api.skyhigh.iik.ntnu.no:9696/v2.0/floatingips', $args);

            // If WordPress error, die with error message
            if ( is_wp_error($response) ) {
                $error_message = $response->get_error_message();
                wp_die($error_message);
            }

            // Decode the response into json
            $decoded = json_decode(wp_remote_retrieve_body($response));
            if ( isset($decoded->error) ) {
                wp_die('Error');
            }

            // Get the floating IP of the newly created floating IP
            $server_floating_ip = $decoded->floatingip->floating_ip_address;


            // Insert the newly created VM into pema_vm
            // MISSING PRIVATE KEY?
            $insert = $wpdb->insert(
                'pema_vm',
                array(
                    'vmServerID' => $server_id,
                    'vmIP' => $server_floating_ip,
                    'vmKey' => $keypair_priv
                ),
                array(
                    '%s',
                    '%s',
                    '%s'
                )
            );

            // If there was an error inserting, die with an error
            if ( !$insert ) {
                // Die with echo message
                wp_die($insert);
            }

            // Get the newly inserted ID
            $vm_get_inserted_id = $wpdb->insert_id;

            // Associate group with the VM
            $insert = $wpdb->insert(
                'pema_group_vm',
                array(
                    'vmID' => $vm_get_inserted_id,
                    'groupID' => $group,
                    'labID' => $labID
                ),
                array(
                    '%d',
                    '%s',
                    '%d'
                )
            );

            // If there was an error inserting into pema_group_vm, die with error message
            if ( !$insert ) {
                wp_die('ErrorIn');
            }
        }
    }
}
// Hook into 'pema_adding_vms_to_groups' and run the function 'pema_plugin_openstack_backend'
// The function has a priority of 10 (default), then specifies arguments sent
add_action('pema_adding_vms_to_groups', 'pema_plugin_openstack_backend', 10, 2);


// Make AJAX able to run the specified function
add_action('wp_ajax_pema_openstack_api_restart_vm', 'pema_openstack_api_restart_vm');
// Function for restarting machines
function pema_openstack_api_restart_vm() {
    // Check the nonce, if its wrong then it will die()
    check_admin_referer('pema_ajax_nonce', 'security');

    // Get the server ID of the server thats gonna restart
    $serverID = $_POST["serverid"];

    // Set the args for restarting machine
    $args = pema_plugin_openstack_api_arg_post('{"reboot": {"type": "hard"}}');
    // Restart the server
    $response = wp_remote_post('https://api.skyhigh.iik.ntnu.no:8774/v2.1/servers/' . $serverID . '/action', $args);
    // If there was an WordPress error running the query, die with error message
    if ( is_wp_error($response) ) {
        $error_message = $response->get_error_message();
        wp_die($error_message);
        // Else, it was successfull
    } else {
        wp_die("Success");
    }
}


// Function for running when the plugin is activated
function pema_plugin_on_activation() {
    // Define an array for inserting a page
    $page = array(
        'post_title' => 'VM Management',
        'post_content' => 'This page allows you to manage your Virtual Machines.',
        'post_status' => 'publish',
        'post_type' => 'page',
        'post_author' => 1,
    );
    // Get the recently inserted page ID
    $get_page_id = wp_insert_post($page);

    // Update the post meta of the page to use a specific template page
    update_post_meta($get_page_id, '_wp_page_template', 'VM-management-template.php');

    global $wpdb;

    // Gets the database charset
    $charset_collate = $wpdb->get_charset_collate();

    $pema_openstack_sql =   "CREATE TABLE IF NOT EXISTS pema_openstack_token (
                                tokenID int(11) NOT NULL AUTO_INCREMENT,
                                tokenToken text,
                                tokenCreated timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                                PRIMARY KEY  (tokenID)
                            ) $charset_collate;";

    // Includes upgrade.php which is used to create/update DB tables
    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta($pema_openstack_sql);
    // If the schedule event doesn't exist
    if (! wp_next_scheduled ( 'pema_openstack_every_three_hours' ) ) {
        // Schedule a function to run every three hours 
        wp_schedule_event(time(), 'three', 'pema_openstack_every_three_hours');
    }
}
// Hook runs on plugin activation
register_activation_hook(__FILE__, 'pema_plugin_on_activation');

// Action that is called every 3 hours, runs the function pema_openstack_create_token()
add_action('pema_openstack_every_three_hours', 'pema_openstack_create_token');

function pema_openstack_create_token() {
    // Initialize DB connection
    global $wpdb;

    // Set arguments for POST call
    $args = array(
        "headers" => array(
            "Content-Type" => "application/json"
        ),
        "timeout" => 60,
        "body" => '{ "auth": {
                "identity": {
                  "methods": ["password"],
                  "password": {
                    "user": {
                      "name": "your-name",
                      "domain": { "id": "domain-id" },
                      "password": "your-password"
                    }
                  }
                },
                "scope": {
                  "project": {
                    "name": "project-name",
                    "domain": { "id": "domain-id" }
                  }
                }
              }
            }'
    );

    // Query a token from the openstack API
    $response = wp_remote_post('https://api.skyhigh.iik.ntnu.no:5000/v3/auth/tokens', $args);

    // If error, die with the message
    if ( is_wp_error($response) ) {
        $error_message = $response->get_error_message();
        wp_die($error_message);
    }

    // Get the header of the returned message
    // This is because x-subject-token is sent in the header
    $decoded = $response["headers"];

    // Get the token
    // offsetGet() is a class function (https://developer.wordpress.org/reference/classes/requests_utility_caseinsensitivedictionary/)
    $token = $decoded->offsetGet('x-subject-token');

    // Replace/Insert the token
    $insert = $wpdb->replace(
        'pema_openstack_token',
        array(
            'tokenID' => 1,
            'tokenToken' => $token,
        ),
        array(
            '%d',
            '%s'
        )
    );
    if ( $insert === false ) {
        // Error...
    }
}


// Function for cleaning up when plugin is deactivated
function pema_plugin_on_deactivation() {
    // Get the object page based on the title
    $page = get_page_by_title('VM Management');
    // Delete the post, true means that it will be permanently deleted instead of just "trashed"
    wp_delete_post($page->ID, true);
    // Clear the scheduled hook
    wp_clear_scheduled_hook('pema_openstack_every_three_hours');
}
// Hook runs on plugin deactivation
register_deactivation_hook(__FILE__, 'pema_plugin_on_deactivation');


// Function that makes a template page available from the plugin
function pema_plugin_add_template( $page_template ) {
    if ( is_page( 'VM Management' ) ) {
        $page_template = dirname( __FILE__ ) . '/VM-management-template.php';
    }
    return $page_template;
}
// When apply_filters('page_template') is run in WP core
// Run the function pema_plugin_add_template, which does something to the $page_template variable
add_filter( 'page_template', 'pema_plugin_add_template' );

// https://wordpress.stackexchange.com/questions/3396/create-custom-page-templates-with-plugins

// Function for adding a three hour filter for WP cron schedules
function pema_openstack_add_schedule( $schedules ) {
    // Create an array with the interval of three hours
    $schedules['three'] = array(
        'interval' => 10800,
        'display' => _('Every three hours')
    );
    // Return $schedules
    return $schedules;
}
// Add new option for scheduling cron in WP (default is hourly, daily, twicedaily)
add_filter('cron_schedules', 'pema_openstack_add_schedule');


?>